@@ -88,7 +88,7 @@ static void init_async_buf( comedi_async *async );
 static int comedi_ioctl(struct inode * inode,struct file * file,
 	unsigned int cmd,unsigned long arg)
 {
-	unsigned int minor=minor(inode->i_rdev);
+	unsigned int minor=MINOR(inode->i_rdev);
 	comedi_device *dev=comedi_get_device_by_minor(minor);
 
 	/* Device config is special, because it must work on
@@ -158,7 +158,7 @@ static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg, unsigned
 	unsigned char *aux_data = NULL;
 	int aux_len;
 
-	if(!suser())
+	if(!capable(CAP_SYS_ADMIN))
 		return -EPERM;
 
 	if(arg==NULL){
@@ -232,7 +232,7 @@ static int do_bufconfig_ioctl(comedi_device *dev,void *arg)
 	}
 
 	if(bc.maximum_size){
-		if(!suser())return -EPERM;
+		if(!capable(CAP_SYS_ADMIN))return -EPERM;
 
 		async->max_bufsize = bc.maximum_size;
 	}
@@ -505,15 +505,20 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 
 	if(bi.bytes_written && s==dev->write_subdev){
 		bi.bytes_written = comedi_buf_write_alloc( async, bi.bytes_written );
-		comedi_buf_munge(dev, s, bi.bytes_written);
+		comedi_buf_munge(dev, s, async->buf_write_alloc_count - async->munge_count);
 		comedi_buf_write_free(async, bi.bytes_written);
 	}
 
 	if(s==dev->read_subdev){
+		unsigned int n_munge_bytes;
+
 		bi.buf_int_count = async->buf_write_count;
 		bi.buf_int_ptr = async->buf_write_ptr;
 		bi.buf_user_count = async->buf_read_count;
 		bi.buf_user_ptr = async->buf_read_ptr;
+
+		n_munge_bytes = bi.buf_int_count - s->async->munge_count;
+		comedi_buf_munge(dev, s, n_munge_bytes);
 	}else{
 		bi.buf_int_count = async->buf_read_count;
 		bi.buf_int_ptr = async->buf_read_ptr;
@@ -699,7 +704,7 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 			ret = -EIO;
 			goto out;
 		}
-	
+
 		/* are we locked? (ioctl lock) */
 		if(s->lock && s->lock!=file){
 			DPRINTK("device locked\n");
@@ -725,6 +730,7 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 				ret=s->insn_read(dev,s,insn,data);
 				break;
 			case INSN_WRITE:
+				//XXX check against subdevice's maxdata
 				ret=s->insn_write(dev,s,insn,data);
 				break;
 			case INSN_BITS:
@@ -924,8 +930,6 @@ static int do_cmd_ioctl(comedi_device *dev,void *arg,void *file)
 		async->cb_mask |= COMEDI_CB_EOS;
 	}
 
-	async->events = 0;
-
 	s->runflags=SRF_USER;
 
 	s->subdev_flags|=SDF_RUNNING;
@@ -1249,7 +1253,7 @@ static struct vm_operations_struct comedi_vm_ops={
 
 static int comedi_mmap_v22(struct file * file, struct vm_area_struct *vma)
 {
-	unsigned int minor=minor(RDEV_OF_FILE(file));
+	unsigned int minor=MINOR(RDEV_OF_FILE(file));
 	comedi_device *dev=comedi_get_device_by_minor(minor);
 	comedi_async *async = NULL;
 	unsigned long start = vma->vm_start;
@@ -1321,7 +1325,7 @@ static unsigned int comedi_poll_v22(struct file *file, poll_table * wait)
 	comedi_async *async;
 	unsigned int mask;
 
-	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1336,7 +1340,7 @@ static unsigned int comedi_poll_v22(struct file *file, poll_table * wait)
 		s = dev->read_subdev;
 		async = s->async;
 		if(!s->busy
-		   || comedi_buf_read_n_available(async)>0
+		   || comedi_buf_read_n_available(s)>0
 		   || !(s->subdev_flags&SDF_RUNNING)){
 			mask |= POLLIN | POLLRDNORM;
 		}
@@ -1346,7 +1350,7 @@ static unsigned int comedi_poll_v22(struct file *file, poll_table * wait)
 		async = s->async;
 		if(!s->busy
 		   || !(s->subdev_flags&SDF_RUNNING)
-		   || comedi_buf_write_n_available(async)>0){
+		   || comedi_buf_write_n_available(s)>0){
 			mask |= POLLOUT | POLLWRNORM;
 		}
 	}
@@ -1363,7 +1367,7 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
 
-	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1425,7 +1429,7 @@ static ssize_t comedi_write_v22(struct file *file,const char *buf,size_t nbytes,
 			n -= m;
 			retval = -EFAULT;
 		}
-		comedi_buf_munge(dev, s, n);
+		comedi_buf_munge(dev, s, async->buf_write_alloc_count - async->munge_count);
 		comedi_buf_write_free(async, n);
 
 		count+=n;
@@ -1449,7 +1453,7 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 	int n,m,count=0,retval=0;
 	DECLARE_WAITQUEUE(wait,current);
 
-	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
 
 	if(!dev->attached)
 	{
@@ -1475,7 +1479,7 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 
 		n=nbytes;
 
-		m = comedi_buf_read_n_available(async);
+		m = comedi_buf_read_n_available(s);
 //printk("%d available\n",m);
 		if(async->buf_read_ptr + m > async->prealloc_bufsz){
 			m = async->prealloc_bufsz - async->buf_read_ptr;
@@ -1505,6 +1509,7 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 			continue;
 		}
 
+		comedi_buf_munge(dev, s, n);
 		m = copy_to_user(buf, async->prealloc_buf +
 			async->buf_read_ptr, n);
 		if(m){
@@ -1567,7 +1572,7 @@ static loff_t comedi_lseek_v22(struct file *file,loff_t offset,int origin)
 	comedi_device *dev;
 	loff_t new_offset;
 
-	dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
+	dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
 
 	switch(origin){
 	case SEEK_SET:
@@ -1590,16 +1595,18 @@ static loff_t comedi_lseek_v22(struct file *file,loff_t offset,int origin)
 
 static int comedi_fop_open(struct inode *inode,struct file *file)
 {
-	unsigned int minor=minor(inode->i_rdev);
+	unsigned int minor=MINOR(inode->i_rdev);
 	comedi_device *dev;
 	char mod[32];
 
-	if(minor>=COMEDI_NDEVICES)return -ENODEV;
-
+	if(minor>=COMEDI_NDEVICES){
+		DPRINTK("invalid minor number\n");
+		return -ENODEV;
+	}
 	dev=comedi_get_device_by_minor(minor);
 
 	/* This is slightly hacky, but we want module autoloading
-	 * to work for root. 
+	 * to work for root.
 	 * case: user opens device, attached -> ok
 	 * case: user opens device, unattached, in_request_module=0 -> autoload
 	 * case: user opens device, unattached, in_request_module=1 -> fail
@@ -1613,9 +1620,11 @@ static int comedi_fop_open(struct inode *inode,struct file *file)
 	 */
 	if(dev->attached)
 		goto ok;
-	if(!suser() && dev->in_request_module)
+	if(!capable(CAP_SYS_ADMIN) && dev->in_request_module){
+		DPRINTK("in request module\n");
 		return -ENODEV;
-	if(suser() && dev->in_request_module)
+	}
+	if(capable(CAP_SYS_ADMIN) && dev->in_request_module)
 		goto ok;
 
 	dev->in_request_module=1;
@@ -1627,10 +1636,10 @@ static int comedi_fop_open(struct inode *inode,struct file *file)
 
 	dev->in_request_module=0;
 
-	if(dev->attached || suser())
-		goto ok;
-	return -ENODEV;
-
+	if(!dev->attached && !capable(CAP_SYS_ADMIN)){
+		DPRINTK("not attached and not root\n");
+		return -ENODEV;
+	}
 ok:
 	if(!try_module_get(THIS_MODULE))
 		return -ENOSYS;
@@ -1650,21 +1659,23 @@ ok:
 
 static int comedi_close_v22(struct inode *inode,struct file *file)
 {
-	comedi_device *dev=comedi_get_device_by_minor(minor(inode->i_rdev));
+	comedi_device *dev=comedi_get_device_by_minor(MINOR(inode->i_rdev));
 	comedi_subdevice *s = NULL;
 	int i;
 
-	for(i=0;i<dev->n_subdevices;i++){
-		s=dev->subdevices+i;
+	if(dev->subdevices)
+	{
+		for(i=0;i<dev->n_subdevices;i++){
+			s=dev->subdevices+i;
 
-		if(s->busy==file){
-			do_cancel(dev,s);
-		}
-		if(s->lock==file){
-			s->lock=NULL;
+			if(s->busy==file){
+				do_cancel(dev,s);
+			}
+			if(s->lock==file){
+				s->lock=NULL;
+			}
 		}
 	}
-
 	if(dev->attached && dev->use_count==1 && dev->close){
 		dev->close(dev);
 	}
@@ -1688,7 +1699,7 @@ static int comedi_close_v22(struct inode *inode,struct file *file)
 #if LINUX_VERSION_CODE >= 0x020100
 static int comedi_fasync (int fd, struct file *file, int on)
 {
-	comedi_device *dev=comedi_get_device_by_minor(minor(RDEV_OF_FILE(file)));
+	comedi_device *dev=comedi_get_device_by_minor(MINOR(RDEV_OF_FILE(file)));
 
 	return fasync_helper(fd,file,on,&dev->async_queue);
 }
@@ -1795,7 +1806,7 @@ static void __exit comedi_cleanup(void)
 	if(MOD_IN_USE)
 		printk("comedi: module in use -- remove delayed\n");
 
-	for(i=0;i<4;i++){
+	for(i=0;i<COMEDI_NDEVICES;i++){
 		char name[20];
 		sprintf(name, "comedi%d", i);
 		devfs_unregister(devfs_find_handle(NULL, name,
@@ -1895,5 +1906,9 @@ static void init_async_buf( comedi_async *async )
 	async->cur_chan = 0;
 	async->scan_progress = 0;
 	async->munge_chan = 0;
+	async->munge_count = 0;
+	async->munge_ptr = 0;
+
+	async->events = 0;
 }
 
