@@ -35,6 +35,7 @@
 #include <linux/comedidev.h>
 #include <linux/wrapper.h>
 #include <linux/highmem.h>  /* for SuSE brokenness */
+#include <linux/vmalloc.h>
 
 #include <asm/io.h>
 
@@ -50,16 +51,40 @@ int comedi_buf_alloc(comedi_device *dev, comedi_subdevice *s,
 
 comedi_driver *comedi_drivers;
 
-int comedi_modprobe(kdev_t minor)
+int comedi_modprobe(int minor)
 {
 	return -EINVAL;
 }
 
-int comedi_device_detach(comedi_device *dev)
+static void cleanup_device_allocations(comedi_device *dev)
 {
 	int i;
 	comedi_subdevice *s;
 
+	if(dev->subdevices)
+	{
+		for(i = 0; i < dev->n_subdevices; i++)
+		{
+			s = dev->subdevices + i;
+			if(s->async)
+			{
+				comedi_buf_alloc(dev, s, 0);
+				if(s->buf_change) s->buf_change(dev, s, 0);
+				kfree(s->async);
+			}
+		}
+		kfree(dev->subdevices);
+		dev->subdevices = NULL;
+	}
+	if(dev->private)
+	{
+		kfree(dev->private);
+		dev->private = NULL;
+	}
+}
+
+int comedi_device_detach(comedi_device *dev)
+{
 	if(!dev->attached)
 		return 0;
 
@@ -68,24 +93,13 @@ int comedi_device_detach(comedi_device *dev)
 
 	dev->attached=0;
 
-	for(i=0;i<dev->n_subdevices;i++){
-		s=dev->subdevices+i;
-		if(s->async){
-			comedi_buf_alloc(dev, s, 0);
-			if(s->buf_change) s->buf_change(dev,s,0);
-			kfree(s->async);
-		}
-	}
-
 	if(dev->driver){
 		dev->driver->detach(dev);
 	}else{
 		printk("BUG: dev->driver=NULL in comedi_device_detach()\n");
 	}
 
-	if(dev->subdevices)kfree(dev->subdevices);
-	if(dev->private)kfree(dev->private);
-
+	cleanup_device_allocations(dev);
 	return 0;
 }
 
@@ -129,8 +143,7 @@ int comedi_device_attach(comedi_device *dev,comedi_devconfig *it)
 		ret=driv->attach(dev,it);
 		if(ret<0){
 			driv->detach(dev);
-			if(dev->subdevices)kfree(dev->subdevices);
-			if(dev->private)kfree(dev->private);
+			cleanup_device_allocations(dev);
 			module_put( driv->module );
 			return ret;
 		}
@@ -157,9 +170,7 @@ attached:
 	if(ret < 0)
 	{
 		driv->detach(dev);
-		if(dev->subdevices)kfree(dev->subdevices);
-		if(dev->private)kfree(dev->private);
-
+		cleanup_device_allocations(dev);
 		return ret;
 	}
 
@@ -459,38 +470,48 @@ int comedi_buf_alloc(comedi_device *dev, comedi_subdevice *s,
 	return 0;
 }
 
-// munging is applied to data right before comedi_buf_write_free()
-void comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
+/* munging is applied to data by core as it passes between user
+ * and kernel space */
+unsigned int comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
 	unsigned int num_bytes )
 {
-	unsigned int offset = s->async->buf_write_ptr;
+	unsigned int count = 0;
 
 	if( s->munge == NULL || ( s->async->cmd.flags & CMDF_RAWDATA ) )
-		return;
+		return count;
 
-	while( num_bytes )
+	/* don't munge partial samples */
+	num_bytes -= num_bytes % bytes_per_sample(s);
+	while( count < num_bytes )
 	{
 		unsigned int block_size;
 
-		block_size = num_bytes;
-		if( (int)(offset + block_size - s->async->prealloc_bufsz) > 0 )
-			block_size = s->async->prealloc_bufsz - offset;
+		block_size = num_bytes - count;
+		if( (int)(s->async->munge_ptr + block_size - s->async->prealloc_bufsz) > 0 )
+			block_size = s->async->prealloc_bufsz - s->async->munge_ptr;
 
-		s->munge( dev, s, s->async->prealloc_buf + offset,
+		s->munge( dev, s, s->async->prealloc_buf + s->async->munge_ptr,
 			block_size, s->async->munge_chan );
 
-		s->async->munge_chan += num_bytes / bytes_per_sample( s );
+		s->async->munge_chan += block_size / bytes_per_sample( s );
 		s->async->munge_chan %= s->async->cmd.chanlist_len;
-		num_bytes -= block_size;
-		offset = 0;
+		s->async->munge_count += block_size;
+		s->async->munge_ptr += block_size;
+		s->async->munge_ptr %= s->async->prealloc_bufsz;
+		count += block_size;
 	}
+	return count;
 }
 
-unsigned int comedi_buf_write_n_available(comedi_async *async)
+unsigned int comedi_buf_write_n_available(comedi_subdevice *s)
 {
+	comedi_async *async=s->async;
 	unsigned int free_end = async->buf_read_count + async->prealloc_bufsz;
+	unsigned int nbytes;
 
-	return free_end - async->buf_write_alloc_count;
+	nbytes = free_end - async->buf_write_alloc_count;
+	nbytes -= nbytes % bytes_per_sample(s);
+	return nbytes;
 }
 
 unsigned int comedi_buf_write_alloc(comedi_async *async, unsigned int nbytes)
@@ -592,16 +613,27 @@ void comedi_buf_memcpy_from(comedi_async *async, unsigned int offset,
 	}
 }
 
-unsigned int comedi_buf_read_n_available(comedi_async *async)
+static inline unsigned int _comedi_buf_read_n_available(comedi_async *async)
 {
-	unsigned int read_end = async->buf_write_count;
+	return async->buf_write_count - async->buf_read_count;
+}
 
-	return read_end - async->buf_read_count;
+unsigned int comedi_buf_read_n_available(comedi_subdevice *s)
+{
+	comedi_async *async = s->async;
+	unsigned int nbytes;
+
+	if(async == NULL)
+		return 0;
+
+	nbytes = _comedi_buf_read_n_available(async);
+	nbytes -= nbytes % bytes_per_sample(s);
+	return nbytes;
 }
 
 int comedi_buf_get(comedi_async *async, sampl_t *x)
 {
-	unsigned int n = comedi_buf_read_n_available(async);
+	unsigned int n = _comedi_buf_read_n_available(async);
 
 	if(n<sizeof(sampl_t))return 0;
 	*x = *(sampl_t *)(async->prealloc_buf + async->buf_read_ptr);
