@@ -26,6 +26,7 @@ Description: Standard PC parallel port
 Author: ds
 Status: works in immediate mode
 Devices: [standard] parallel port (comedi_parport)
+Updated: Tue, 30 Apr 2002 21:11:45 -0700
 
 A cheap and easy way to get a few more digital I/O lines.  Steal
 additional parallel ports from old computers or your neighbors'
@@ -60,11 +61,11 @@ pin     subdev  chan    aka
 
 Notes:
 
-Channel 0 and 2 are output, channel 1 is input.  I know that it
-is possible to change this with ECP/EPP parallel ports, but this
-driver is a cheap hack.
+Subdevices 0 is digital I/O, subdevice 1 is digital input, and
+subdevice 2 is digital output.  Unlike other Comedi devices,
+subdevice 0 defaults to output.
 
-Pins 13 and 14 are inverted once by comedi and once by the
+Pins 13 and 14 are inverted once by Comedi and once by the
 hardware, thus cancelling the effect.
 
 Pin 1 is a strobe, thus acts like one.  There's no way in software
@@ -72,31 +73,16 @@ to change this, at least on a standard parallel port.
 
 Subdevice 3 pretends to be a digital input subdevice, but it always
 returns 0 when read.  However, if you run a command with
-scan_begin_src=TRIG_EXT, it uses pin 13 as a external triggering
-pin, which can be used to wake up tasks.  (or is that pin 10. --ds)
+scan_begin_src=TRIG_EXT, it uses pin 10 as a external triggering
+pin, which can be used to wake up tasks.
 */
 /*
-
-   TODO:
-
-   - EPP/ECP support
-
    see http://www.beyondlogic.org/ for information.
+   or http://www.linux-magazin.de/ausgabe/1999/10/IO/io.html
  */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
+#include <linux/ioport.h>
 
 
 #define PARPORT_SIZE 3
@@ -133,11 +119,26 @@ static int parport_insn_a(comedi_device *dev,comedi_subdevice *s,
 		outb(devpriv->a_data,dev->iobase+PARPORT_A);
 	}
 
-	data[1] = devpriv->a_data;
+	data[1] = inb(devpriv->a_data);
 
 	return 2;
 }
 	
+static int parport_insn_config_a(comedi_device *dev,comedi_subdevice *s,
+	comedi_insn *insn,lsampl_t *data)
+{
+	if(data[0]){
+		s->io_bits = 0xff;
+		devpriv->c_data &= ~(1<<5);
+	}else{
+		s->io_bits = 0;
+		devpriv->c_data |= (1<<5);
+	}
+	outb(devpriv->c_data,dev->iobase+PARPORT_C);
+
+	return 1;
+}
+	
 static int parport_insn_b(comedi_device *dev,comedi_subdevice *s,
 	comedi_insn *insn,lsampl_t *data)
 {
@@ -169,6 +170,8 @@ static int parport_insn_c(comedi_device *dev,comedi_subdevice *s,
 static int parport_intr_insn(comedi_device *dev,comedi_subdevice *s,
 	comedi_insn *insn,lsampl_t *data)
 {
+	if(insn->n<1)return -EINVAL;
+
 	data[1] = 0;
 	return 2;
 }
@@ -261,25 +264,21 @@ static int parport_intr_cancel(comedi_device *dev,comedi_subdevice *s)
 	return 0;
 }
 
-static void parport_interrupt(int irq,void *d,struct pt_regs *regs)
+static irqreturn_t parport_interrupt(int irq,void *d,struct pt_regs *regs)
 {
 	comedi_device *dev=d;
 	comedi_subdevice *s=dev->subdevices+3;
 
 	if(!devpriv->enable_irq){
 		printk("comedi_parport: bogus irq, ignored\n");
-		return;
+		return IRQ_NONE;
 	}
 
-	*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=0;
-	s->async->buf_int_ptr+=sizeof(sampl_t);
-	s->async->buf_int_count+=sizeof(sampl_t);
-	if(s->async->buf_int_ptr>=s->async->data_len){
-		s->async->buf_int_ptr=0;
-		s->async->events |= COMEDI_CB_EOBUF;
-	}
-	
+	comedi_buf_put( s->async, 0 );
+	s->async->events |= COMEDI_CB_EOS;
+
 	comedi_event(dev,s,s->async->events);
+	return IRQ_HANDLED;
 }
 
 static int parport_attach(comedi_device *dev,comedi_devconfig *it)
@@ -311,19 +310,19 @@ static int parport_attach(comedi_device *dev,comedi_devconfig *it)
 	}
 	dev->board_name="parport";
 
-	dev->n_subdevices=4;
-	if((ret=alloc_subdevices(dev))<0)
+	if((ret=alloc_subdevices(dev, 4))<0)
 		return ret;
 	if((ret=alloc_private(dev,sizeof(parport_private)))<0)
 		return ret;
 
 	s=dev->subdevices+0;
-	s->type=COMEDI_SUBD_DO;
-	s->subdev_flags=SDF_WRITEABLE;
+	s->type=COMEDI_SUBD_DIO;
+	s->subdev_flags=SDF_READABLE|SDF_WRITABLE;
 	s->n_chan=8;
 	s->maxdata=1;
 	s->range_table=&range_digital;
 	s->insn_bits = parport_insn_a;
+	s->insn_config = parport_insn_config_a;
 
 	s=dev->subdevices+1;
 	s->type=COMEDI_SUBD_DI;
@@ -335,7 +334,7 @@ static int parport_attach(comedi_device *dev,comedi_devconfig *it)
 
 	s=dev->subdevices+2;
 	s->type=COMEDI_SUBD_DO;
-	s->subdev_flags=SDF_WRITEABLE;
+	s->subdev_flags=SDF_WRITABLE;
 	s->n_chan=4;
 	s->maxdata=1;
 	s->range_table=&range_digital;
