@@ -27,7 +27,7 @@
 /*
 Driver: comedi_rt_timer.o
 Description: Command emulator using real-time tasks
-Authors: ds, fmhess
+Author: ds, fmhess
 Devices:
 Status: works
 
@@ -42,27 +42,22 @@ higher than with a device that has an on-board timer, and maximum
 sample rate is much lower.
 
 Configuration options:
-  [0] - device index
-  [1] - subdevice index
+  [0] - minor number of device you wish to emulate commands for
+  [1] - subdevice number you wish to emulate commands for
 */
 /*
-Options:
-	[0] - minor number of device you wish to emulate commands for
-	[1] - subdevice number you wish to emulate commands for
-
 TODO:
 	Support for digital io commands could be added, except I can't see why
 		anyone would want to use them
 	What happens if device we are emulating for is de-configured?
-
 */
 
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
 #include <linux/comedilib.h>
+
+#include "comedi_fc.h"
+
 #ifdef CONFIG_COMEDI_RTL_V1
 #include <rtl_sched.h>
 #include <asm/rt_irq.h>
@@ -71,6 +66,8 @@ TODO:
 #include <rtl.h>
 #include <rtl_sched.h>
 #include <rtl_compat.h>
+#include <asm/div64.h>
+
 
 #ifndef RTLINUX_VERSION_CODE
 #define RTLINUX_VERSION_CODE 0
@@ -84,26 +81,7 @@ TODO:
 // this function sole purpose is to divide a long long by 838
 static inline RTIME nano2count(long long ns)
 {
-	unsigned long denom = 838;	// divisor
-	long ms32 = ns >> 32;	// most significant 32 bits
-	unsigned long ms32rem = ms32 % denom;	// remainder of ms32 / denom
-	unsigned long ls32 = ns & 0xffffffff;	// least significant 32 bits
-	unsigned long ls32rem = ls32 % denom;
-	unsigned long big = 0xffffffff;
-	unsigned long big_rem = big % denom;
-	unsigned long rem_rem;
-
-	// divide most significant bits
-	ns = ms32 / denom;
-	ns = ns << 32;
-	// add corrections due to rounding errors
-	ns += ms32rem * (big / denom) + (ms32rem * (big_rem + 1)) / denom;
-	// divide least significant bits
-	ns += ls32 / denom;
-	// add really small correction
-	rem_rem = (ms32rem * (big_rem + 1)) % denom;
-	ns += (ls32rem + rem_rem) / denom;
-
+	do_div(ns, 838);
 	return ns;
 }
 #ifdef rt_get_time()
@@ -124,6 +102,9 @@ static inline RTIME nano2count(long long ns)
 #define rt_request_srq(x,y,z) rtl_get_soft_irq(y,"timer")
 #define rt_task_init(a,b,c,d,e,f,g) rt_task_init(a,b,c,d,(e)+1)
 #define rt_task_resume(x) rt_task_wakeup(x)
+#define rt_set_oneshot_mode()
+#define start_rt_timer(x)
+#define stop_rt_timer()
 
 #endif
 #ifdef CONFIG_COMEDI_RTAI
@@ -147,13 +128,14 @@ static comedi_driver driver_timer={
 	driver_name:	"comedi_rt_timer",
 	attach:		timer_attach,
 	detach:		timer_detach,
+//	open:		timer_open,
 };
 COMEDI_INITCLEANUP(driver_timer);
 
 
 typedef struct{
-	int device;	// device we are emulating commands for
-	int subd;	// subdevice we are emulating commands for
+	comedi_t *device;	// device we are emulating commands for
+	int subd;		// subdevice we are emulating commands for
 	RT_TASK *rt_task;	// rt task that starts scans
 	RT_TASK *scan_task;	// rt task that controls conversion timing in a scan
 	/* io_function can point to either an input or output function
@@ -170,6 +152,7 @@ typedef struct{
 	volatile int stop;	// indicates we should stop
 	volatile int rt_task_active;	// indicates rt_task is servicing a comedi_cmd
 	volatile int scan_task_active;	// indicates scan_task is servicing a comedi_cmd
+	unsigned timer_running : 1;
 }timer_private;
 #define devpriv ((timer_private *)dev->private)
 
@@ -231,7 +214,11 @@ static int timer_data_read(comedi_device *dev, comedi_cmd *cmd,
 		comedi_error(dev, "read error");
 		return -EIO;
 	}
-	comedi_buf_put(s->async, data);
+	if( s->flags & SDF_LSAMPL ){
+		cfc_write_long_to_buffer( s, data );
+	}else{
+		comedi_buf_put( s->async, data );
+	}
 
 	return 0;
 }
@@ -241,11 +228,20 @@ static int timer_data_write(comedi_device *dev, comedi_cmd *cmd,
 	unsigned int index)
 {
 	comedi_subdevice *s = dev->write_subdev;
-	int ret;
+	unsigned int num_bytes;
 	sampl_t data;
+	lsampl_t long_data;
+	int ret;
 
-	ret = comedi_buf_get(s->async, &data);
-	if(ret < 0) {
+	if( s->flags & SDF_LSAMPL ) {
+		num_bytes = cfc_read_array_from_buffer( s, &long_data, sizeof( long_data ) );
+	}else{
+		num_bytes = cfc_read_array_from_buffer( s, &data, sizeof( data ) );
+		long_data = data;
+	}
+
+	if(num_bytes == 0)
+	{
 		comedi_error(dev, "buffer underrun");
 		return -EAGAIN;
 	}
@@ -253,7 +249,7 @@ static int timer_data_write(comedi_device *dev, comedi_cmd *cmd,
 		CR_CHAN(cmd->chanlist[index]),
 		CR_RANGE(cmd->chanlist[index]),
 		CR_AREF(cmd->chanlist[index]),
-		data);
+		long_data);
 	if(ret<0){
 		comedi_error(dev, "write error");
 		return -EIO;
@@ -275,7 +271,11 @@ static int timer_dio_read(comedi_device *dev, comedi_cmd *cmd,
 		comedi_error(dev, "read error");
 		return -EIO;
 	}
-	comedi_buf_put(s->async, data);
+
+	if( s->flags & SDF_LSAMPL )
+		cfc_write_long_to_buffer( s, data );
+	else
+		cfc_write_to_buffer( s, data );
 
 	return 0;
 }
@@ -564,7 +564,7 @@ static int timer_start_cmd(comedi_device *dev, comedi_subdevice *s)
 	comedi_cmd *cmd = &async->cmd;
 	RTIME now, delay, period;
 	int ret;
-	
+
 	devpriv->stop = 0;
 	s->async->events = 0;
 
@@ -587,17 +587,9 @@ static int timer_start_cmd(comedi_device *dev, comedi_subdevice *s)
 		comedi_error(dev, "error starting rt_task");
 		return ret;
 	}
-
 	return 0;
 }
 
-static int timer_poll(comedi_device *dev,comedi_subdevice *s)
-{
-	/* This it pretty generic code, perhaps it should be the
-	 * default for any subdevice that doesn't implement poll. */
-	return s->async->buf_int_count - s->async->buf_user_count;
-}
-
 static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	int ret;
@@ -606,29 +598,30 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 	/* These should probably be devconfig options[] */
 	const int timer_priority = 4;
 	const int scan_priority = timer_priority + 1;
+	char path[20];
 
 	printk("comedi%d: timer: ",dev->minor);
 
 	dev->board_name="timer";
 
-	dev->n_subdevices=1;
-	if((ret=alloc_subdevices(dev))<0)
+	if((ret=alloc_subdevices(dev, 1))<0)
 		return ret;
 	if((ret=alloc_private(dev,sizeof(timer_private)))<0)
 		return ret;
 
-	devpriv->device=it->options[0];
+	sprintf(path,"/dev/comedi%d",it->options[0]);
+	devpriv->device = comedi_open(path);
 	devpriv->subd=it->options[1];
 
-	printk("device %d, subdevice %d\n", devpriv->device, devpriv->subd);
+	printk("emulating commands for minor %i, subdevice %d\n", it->options[0], devpriv->subd);
 
-	emul_dev=comedi_get_device_by_minor(devpriv->device);
-	emul_s=emul_dev->subdevices+devpriv->subd;
+	emul_dev = devpriv->device;
+	emul_s = emul_dev->subdevices+devpriv->subd;
 
 	// input or output subdevice
 	s=dev->subdevices+0;
 	s->type=emul_s->type;
-	s->subdev_flags = emul_s->subdev_flags;
+	s->subdev_flags = emul_s->subdev_flags; /* SDF_GROUND (to fool check_driver) */
 	s->n_chan=emul_s->n_chan;
 	s->len_chanlist=1024;
 	s->do_cmd=timer_cmd;
@@ -637,7 +630,6 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 	s->maxdata=emul_s->maxdata;
 	s->range_table=emul_s->range_table;
 	s->range_table_list=emul_s->range_table_list;
-	s->poll=timer_poll;
 	switch(emul_s->type){
 	case COMEDI_SUBD_AI:
 		s->insn_read=timer_insn;
@@ -662,9 +654,9 @@ static int timer_attach(comedi_device *dev,comedi_devconfig *it)
 		return -EINVAL;
 	}
 
-#ifdef CONFIG_COMEDI_RTAI
 	rt_set_oneshot_mode();
-#endif
+	start_rt_timer( 1 );
+	devpriv->timer_running = 1;
 
 	devpriv->rt_task = kmalloc(sizeof(RT_TASK),GFP_KERNEL);
 	memset(devpriv->rt_task,0,sizeof(RT_TASK));
@@ -708,6 +700,8 @@ static int timer_detach(comedi_device *dev)
 			rt_task_delete(devpriv->scan_task);
 			kfree(devpriv->scan_task);
 		}
+		if( devpriv->timer_running )
+			stop_rt_timer();
 	}
 
 	return 0;
