@@ -1,6 +1,6 @@
 /*
     das800.c driver for Keitley das800 series boards and compatibles
-    Copyright (C) 2000 Frank Mori Hess <fmhess@uiuc.edu>
+    Copyright (C) 2000 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
     Copyright (C) 2000 David A. Schleef <ds@schleef.org>
@@ -24,7 +24,7 @@
 /*
 Driver: das800.o
 Description: Keithley Metrabyte DAS800 (& compatibles)
-Author: Frank Mori Hess <fmhess@uiuc.edu>
+Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Devices: [Keithley Metrabyte] DAS-800 (das-800), DAS-801 (das-801),
   DAS-802 (das-802),
   [Measurement Computing] CIO-DAS800 (cio-das800),
@@ -35,34 +35,10 @@ Status: works, cio-das802/16 untested - email me if you have tested it
 Configuration options:
   [0] - I/O port base address
   [1] - IRQ (optional, required for timed or externally triggered conversions)
-*/
-/*
 
-This driver supports the Keithley boards:
-das-800
-das-801
-das-802
+Notes:
+	IRQ can be omitted, although the cmd interface will not work without it.
 
-and Measurement Computing (Computer Boards) models:
-cio-das800
-cio-das801
-cio-das802
-cio-das802/16
-
-Options:
-	[0] - base io address
-	[1] - irq
-
-irq can be omitted, although the cmd interface will not work without it.
-
-cmd triggers supported:
-	start_src:      TRIG_NOW | TRIG_EXT
-	scan_begin_src: TRIG_FOLLOW
-	scan_end_src:   TRIG_COUNT
-	convert_src:    TRIG_TIMER | TRIG_EXT
-	stop_src:       TRIG_NONE | TRIG_COUNT
-
-NOTES:
 	All entries in the channel/gain list must use the same gain and be
 	consecutive channels counting upwards in channel number (these are
 	hardware limitations.)
@@ -72,23 +48,26 @@ NOTES:
 
 	The cio-das802/16 does not have a fifo-empty status bit!  Therefore
 	only fifo-half-full transfers are possible with this card.
+*/
+/*
+
+cmd triggers supported:
+	start_src:      TRIG_NOW | TRIG_EXT
+	scan_begin_src: TRIG_FOLLOW
+	scan_end_src:   TRIG_COUNT
+	convert_src:    TRIG_TIMER | TRIG_EXT
+	stop_src:       TRIG_NONE | TRIG_COUNT
+
 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
+#include <linux/comedidev.h>
+
 #include <linux/ioport.h>
 #include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
-#include <linux/comedidev.h>
+
 #include "8253.h"
+#include "comedi_fc.h"
 
 #define DAS800_SIZE           8
 #define TIMER_BASE            1000
@@ -276,7 +255,7 @@ static comedi_driver driver_das800={
 	offset:		sizeof(das800_board),
 };
 
-static void das800_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs);
 static void enable_das800(comedi_device *dev);
 static void disable_das800(comedi_device *dev);
 static int das800_ai_do_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd);
@@ -366,7 +345,7 @@ static int das800_probe(comedi_device *dev)
 COMEDI_INITCLEANUP(driver_das800);
 
 /* interrupt service routine */
-static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t das800_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	short i;		/* loop index */
 	sampl_t dataPoint = 0;
@@ -382,10 +361,10 @@ static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	status = inb(dev->iobase + DAS800_STATUS);
 	/* if interrupt was not generated by board or driver not attached, quit */
-	if(!(status & IRQ) || !(dev->attached))
-	{
-		return;
-	}
+	if(!(status & IRQ) )
+		return IRQ_NONE;
+	if(!(dev->attached))
+		return IRQ_HANDLED;
 
 	/* wait until here to initialize async, since we will get null dereference
 	 * if interrupt occurs before driver is fully attached!
@@ -400,7 +379,7 @@ static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
 	if(status == 0)
 	{
 		comedi_spin_unlock_irqrestore(&dev->spinlock, irq_flags);
-		return;
+		return IRQ_HANDLED;
 	}
 
 	/* loop while card's fifo is not empty (and limit to half fifo for cio-das802/16) */
@@ -429,7 +408,7 @@ static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
 		if(devpriv->count > 0 || devpriv->forever == 1)
 		{
 			/* write data point to buffer */
-			comedi_buf_put(async, dataPoint);
+			cfc_write_to_buffer( s, dataPoint);
 			if(devpriv->count > 0) devpriv->count--;
 		}
 	}
@@ -451,7 +430,7 @@ static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
 		comedi_event(dev, s, async->events);
 		async->events = 0;
-		return;
+		return IRQ_HANDLED;
 	}
 	if(devpriv->count > 0 || devpriv->forever == 1)
 	{
@@ -468,7 +447,7 @@ static void das800_interrupt(int irq, void *d, struct pt_regs *regs)
 	comedi_spin_unlock_irqrestore(&dev->spinlock, irq_flags);
 	comedi_event(dev, s, async->events);
 	async->events = 0;
-	return;
+	return IRQ_HANDLED;
 }
 
 static int das800_attach(comedi_device *dev, comedi_devconfig *it)
@@ -531,8 +510,7 @@ static int das800_attach(comedi_device *dev, comedi_devconfig *it)
 
 	dev->board_name = thisboard->name;
 
-	dev->n_subdevices = 3;
-	if(alloc_subdevices(dev) < 0)
+	if(alloc_subdevices(dev, 3) < 0)
 		return -ENOMEM;
 
 	/* analog input subdevice */
@@ -561,7 +539,7 @@ static int das800_attach(comedi_device *dev, comedi_devconfig *it)
 	/* do */
 	s = dev->subdevices + 2;
 	s->type=COMEDI_SUBD_DO;
-	s->subdev_flags = SDF_WRITEABLE | SDF_READABLE;
+	s->subdev_flags = SDF_WRITABLE | SDF_READABLE;
 	s->n_chan = 4;
 	s->maxdata = 1;
 	s->range_table = &range_digital;
@@ -851,7 +829,7 @@ static int das800_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn
 	range &= 0xf;
 	outb(range, dev->iobase + DAS800_GAIN);
 
-	udelay(5);
+	comedi_udelay(5);
 
 	for(n = 0; n < insn->n; n++)
 	{
