@@ -1,45 +1,64 @@
 /*
- * comedi/drivers/icp_multi.c
- *
- * Author: Anne Smorthit <anne.smorthit@sfwte.ch>
- *
- *
- *  hardware driver for Inova cards:
- *   card:   ICP_MULTI
- *   driver: icp_multi
- *
- * Options:
- *  [0] - PCI bus number - if bus number and slot number are 0, 
- *                         then driver search for first unused card
- *  [1] - PCI slot number 
- * 
+    comedi/drivers/icp_multi.c
+
+    COMEDI - Linux Control and Measurement Device Interface
+    Copyright (C) 1997-2002 David A. Schleef <ds@schleef.org>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
 */
+
+
 /*
 Driver: icp_multi.o
-Description: Inova ICP Multi
+Description: Inova ICP_MULTI
 Author: Anne Smorthit <anne.smorthit@sfwte.ch>
-Devices: [Inova] ICP Multi (icp_multi)
+Devices: [Inova] ICP_MULTI (icp_multi)
+Status: works
 
+The driver works for analog input and output and digital input and output.
+It does not work with interrupts or with the counters.  Currently no support
+for DMA.
+
+It has 16 single-ended or 8 differential Analogue Input channels with 12-bit
+resolution.  Ranges : 5V, 10V, +/-5V, +/-10V, 0..20mA and 4..20mA.  Input
+ranges can be individually programmed for each channel.  Voltage or current
+measurement is selected by jumper.
+
+There are 4 x 12-bit Analogue Outputs.  Ranges : 5V, 10V, +/-5V, +/-10V
+
+16 x Digital Inputs, 24V
+
+8 x Digital Outputs, 24V, 1A
+
+4 x 16-bit counters
+
+Options:
+ [0] - PCI bus number - if bus number and slot number are 0, 
+                        then driver search for first unused card
+ [1] - PCI slot number 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <linux/pci.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
+
+#include <linux/delay.h>
+#include <linux/pci.h>
+
 #include "icp_multi.h"
 
 
-#define VENDOR_ID	0x104C	/* PCI vendor ID */
 #define DEVICE_ID	0x8000	/* Device ID */
 
 #define ICP_MULTI_EXTDEBUG
@@ -62,7 +81,7 @@ Devices: [Inova] ICP Multi (icp_multi)
 #define ICP_MULTI_CNTR2		0x14	/* R/W:	Counter 2 */
 #define ICP_MULTI_CNTR3		0x16	/* R/W:	Counter 3 */
 
-#define ICP_MULTI_PCI_MEM_WINDOW_SIZE		0x10000		/* 64K */
+#define ICP_MULTI_SIZE		0x20	/* 32 bytes */
 
 // Define bits from ADC command/status register
 #define	ADC_ST		0x0001		/* Start ADC */
@@ -92,7 +111,7 @@ Devices: [Inova] ICP Multi (icp_multi)
 
 
 // Define analogue range
-comedi_lrange range_analog={ 4, {
+static comedi_lrange range_analog={ 4, {
 	UNI_RANGE(5),
 	UNI_RANGE(10),
 	BIP_RANGE(5),
@@ -121,7 +140,6 @@ static unsigned short	pci_list_builded=0;	/*=1 list of card is know */
 
 typedef struct {
 	char 		*name;		// driver name
-	int		vendor_id;	// PCI vendor a device ID of card
 	int		device_id;
 	int		iorange;	// I/O range len
 	char		have_irq;	// 1=card support IRQ
@@ -142,7 +160,6 @@ typedef struct {
 static boardtype boardtypes[] =
 {
 	{"icp_multi",		// Driver name
-	 VENDOR_ID,		// PCI vendor ID
 	 DEVICE_ID,		// PCI device ID
 	 IORANGE_ICP_MULTI,	// I/O range length
 	 1,			// 1=Card supports interrupts
@@ -169,33 +186,26 @@ static comedi_driver driver_icp_multi={
 	detach:		icp_multi_detach,
 	num_names:	n_boardtypes,
 	board_name:	boardtypes,
-	offset:		sizeof(boardtype),
+	offset:		sizeof(boardtype),	
 };
+COMEDI_INITCLEANUP(driver_icp_multi);
 
 typedef struct{
 	char			valid;			// card is usable
-	char			neverending_ai;		// we do unlimited AI
+	void			*io_addr;		// Pointer to mapped io address
+	unsigned long		phys_iobase;		// Physical io address
 	unsigned int		AdcCmdStatus;		// ADC Command/Status register
 	unsigned int		DacCmdStatus;		// DAC Command/Status register
 	unsigned int		IntEnable;		// Interrupt Enable register
 	unsigned int		IntStatus;		// Interrupt Status register
-	unsigned int		ai_do;			// what do AI? 0=nothing, 1 to 4 mode
-	unsigned int		ai_act_scan;		// how many scans we finished
-	unsigned int		ai_act_chan;		// actual position in actual scan
-	unsigned int 		ai_buf_ptr;		// data buffer ptr in samples
-	unsigned char		ai_eos;			// 1=EOS wake up
 	unsigned int		act_chanlist[32];	// list of scaned channel
 	unsigned char		act_chanlist_len;	// len of scanlist
 	unsigned char 		act_chanlist_pos;	// actual position in MUX list
-	unsigned int		ai_scans;		// len of scanlist
-	unsigned int		ai_n_chan;		// how many channels is measured	
 	unsigned int		*ai_chanlist;		// actaul chanlist
-	unsigned int		ai_flags;		// flaglist
-	unsigned int		ai_data_len;		// len of data buffer
 	sampl_t			*ai_data;		// data buffer
 	sampl_t			ao_data[4];		// data output buffer
 	sampl_t			di_data;		// Digital input data
-	sampl_t			do_data;		// Digital output data
+	unsigned int		do_data;		// Remember digital output data
 } icp_multi_private;
 
 #define devpriv ((icp_multi_private *)dev->private)
@@ -207,8 +217,10 @@ typedef struct{
 ==============================================================================
 */
 
-int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan);
-void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan);
+#if 0
+static int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan);
+#endif
+static void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan);
 static int icp_multi_reset(comedi_device *dev);
 
 
@@ -237,56 +249,56 @@ static int icp_multi_reset(comedi_device *dev);
 
 ==============================================================================
 */
-int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
 {
 	int n,timeout;
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: BGN: icp_multi_insn_read_ai(...)\n");
+	printk("icp multi EDBG: BGN: icp_multi_insn_read_ai(...)\n");
 #endif
 	// Disable A/D conversion ready interrupt
 	devpriv->IntEnable &= ~ADC_READY;
-	outw(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
+	writew(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
 	
 	// Clear interrupt status
 	devpriv->IntStatus |= ADC_READY;
-	outw(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
+	writew(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
 
 	// Set up appropriate channel, mode and range data, for specified channel
 	setup_channel_list(dev, s, &insn->chanspec, 1);
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp_multi A ST=%4x IO=%x\n",inw(dev->iobase+ICP_MULTI_ADC_CSR), dev->iobase+ICP_MULTI_ADC_CSR);
+	printk("icp_multi A ST=%4x IO=%x\n",readw(dev->iobase+ICP_MULTI_ADC_CSR), dev->iobase+ICP_MULTI_ADC_CSR);
 #endif
 
 	for (n=0; n<insn->n; n++) {
 		// Set start ADC bit
 		devpriv->AdcCmdStatus |= ADC_ST;
-		outw(devpriv->AdcCmdStatus, dev->iobase+ICP_MULTI_ADC_CSR);
+		writew(devpriv->AdcCmdStatus, dev->iobase+ICP_MULTI_ADC_CSR);
 		devpriv->AdcCmdStatus &= ~ADC_ST;
 
 #ifdef ICP_MULTI_EXTDEBUG
-    		rt_printk("icp multi B n=%d ST=%4x\n",n,inw(dev->iobase+ICP_MULTI_ADC_CSR));
+    		printk("icp multi B n=%d ST=%4x\n",n,readw(dev->iobase+ICP_MULTI_ADC_CSR));
 #endif
 
-		udelay(1);
+		comedi_udelay(1);
 
 #ifdef ICP_MULTI_EXTDEBUG
-    		rt_printk("icp multi C n=%d ST=%4x\n",n,inw(dev->iobase+ICP_MULTI_ADC_CSR));
+    		printk("icp multi C n=%d ST=%4x\n",n,readw(dev->iobase+ICP_MULTI_ADC_CSR));
 #endif
 
 		// Wait for conversion to complete, or get fed up waiting
     		timeout=100;
     		while (timeout--) {
-			if (!(inw(dev->iobase+ICP_MULTI_ADC_CSR) & ADC_BSY))
+			if (!(readw(dev->iobase+ICP_MULTI_ADC_CSR) & ADC_BSY))
 				goto conv_finish;
 
 #ifdef ICP_MULTI_EXTDEBUG
 			if (!(timeout%10))
-				rt_printk("icp multi D n=%d tm=%d ST=%4x\n",n,timeout,inw(dev->iobase+ICP_MULTI_ADC_CSR));
+				printk("icp multi D n=%d tm=%d ST=%4x\n",n,timeout,readw(dev->iobase+ICP_MULTI_ADC_CSR));
 #endif
 
-			udelay(1);
+			comedi_udelay(1);
     		}
 
 		// If we reach here, a timeout has occurred
@@ -294,34 +306,34 @@ int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, comedi_ins
 
 		// Disable interrupt
 		devpriv->IntEnable &= ~ADC_READY;
-		outw(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
+		writew(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
 
 		// Clear interrupt status
 		devpriv->IntStatus |= ADC_READY;
-		outw(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
+		writew(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
 
 		// Clear data received
     		data[n]=0;
 
 #ifdef ICP_MULTI_EXTDEBUG
-		rt_printk("icp multi EDBG: END: icp_multi_insn_read_ai(...) n=%d\n",n);
+		printk("icp multi EDBG: END: icp_multi_insn_read_ai(...) n=%d\n",n);
 #endif
     		return -ETIME;
 
 conv_finish:
-		data[n] = (inw(dev->iobase+ICP_MULTI_AI) >> 4 ) & 0x0fff;
+		data[n] = (readw(dev->iobase+ICP_MULTI_AI) >> 4 ) & 0x0fff;
 	}
 	
 	// Disable interrupt
 	devpriv->IntEnable &= ~ADC_READY;
-	outw(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
+	writew(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
 
 	// Clear interrupt status
 	devpriv->IntStatus |= ADC_READY;
-	outw(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
+	writew(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
 
 #ifdef ICP_MULTI_EXTDEBUG
-		rt_printk("icp multi EDBG: END: icp_multi_insn_read_ai(...) n=%d\n",n);
+		printk("icp multi EDBG: END: icp_multi_insn_read_ai(...) n=%d\n",n);
 #endif
 	return n;
 }
@@ -344,20 +356,20 @@ conv_finish:
 
 ==============================================================================
 */
-int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
 {
 	int n, chan, range, timeout;
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: BGN: icp_multi_insn_write_ao(...)\n");
+	printk("icp multi EDBG: BGN: icp_multi_insn_write_ao(...)\n");
 #endif
 	// Disable D/A conversion ready interrupt
 	devpriv->IntEnable &= ~DAC_READY;
-	outw(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
+	writew(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
 	
 	// Clear interrupt status
 	devpriv->IntStatus |= DAC_READY;
-	outw(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
+	writew(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
 
 	// Get channel number and range
 	chan = CR_CHAN(insn->chanspec);
@@ -372,21 +384,21 @@ int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, comedi_in
 	devpriv->DacCmdStatus |= this_board->rangecode[range];
 	devpriv->DacCmdStatus |= (chan << 8);
 	
-	outw(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
+	writew(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
 
 	for (n=0; n<insn->n; n++) {
 		// Wait for analogue output data register to be ready for new data, or get fed up waiting
     		timeout=100;
     		while (timeout--) {
-			if (!(inw(dev->iobase+ICP_MULTI_DAC_CSR) & DAC_BSY))
+			if (!(readw(dev->iobase+ICP_MULTI_DAC_CSR) & DAC_BSY))
 				goto dac_ready;
 
 #ifdef ICP_MULTI_EXTDEBUG
 			if (!(timeout%10))
-				rt_printk("icp multi A n=%d tm=%d ST=%4x\n",n,timeout,inw(dev->iobase+ICP_MULTI_DAC_CSR));
+				printk("icp multi A n=%d tm=%d ST=%4x\n",n,timeout,readw(dev->iobase+ICP_MULTI_DAC_CSR));
 #endif
 
-			udelay(1);
+			comedi_udelay(1);
     		}
 
 		// If we reach here, a timeout has occurred
@@ -394,27 +406,27 @@ int icp_multi_insn_write_ao(comedi_device * dev, comedi_subdevice * s, comedi_in
 
 		// Disable interrupt
 		devpriv->IntEnable &= ~DAC_READY;
-		outw(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
+		writew(devpriv->IntEnable,dev->iobase + ICP_MULTI_INT_EN);
 
 		// Clear interrupt status
 		devpriv->IntStatus |= DAC_READY;
-		outw(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
+		writew(devpriv->IntStatus,dev->iobase + ICP_MULTI_INT_STAT);
 
 		// Clear data received
     		devpriv->ao_data[chan]=0;
 
 #ifdef ICP_MULTI_EXTDEBUG
-		rt_printk("icp multi EDBG: END: icp_multi_insn_write_ao(...) n=%d\n",n);
+		printk("icp multi EDBG: END: icp_multi_insn_write_ao(...) n=%d\n",n);
 #endif
     		return -ETIME;
 
 dac_ready:
 		// Write data to analogue output data register
-		outw(data[n], dev->iobase + ICP_MULTI_AO);
+		writew(data[n], dev->iobase + ICP_MULTI_AO);
 
 		// Set DAC_ST bit to write the data to selected channel
 		devpriv->DacCmdStatus |= DAC_ST;
-		outw(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
+		writew(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
 		devpriv->DacCmdStatus &= ~DAC_ST;
 
 		// Save analogue output data
@@ -422,7 +434,7 @@ dac_ready:
 	}
 
 #ifdef ICP_MULTI_EXTDEBUG
-		rt_printk("icp multi EDBG: END: icp_multi_insn_write_ao(...) n=%d\n",n);
+		printk("icp multi EDBG: END: icp_multi_insn_write_ao(...) n=%d\n",n);
 #endif
 		return n;
 }
@@ -445,7 +457,7 @@ dac_ready:
 
 ==============================================================================
 */
-int icp_multi_insn_read_ao(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_read_ao(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
 {
 	int n,chan;
 
@@ -477,9 +489,9 @@ int icp_multi_insn_read_ao(comedi_device * dev, comedi_subdevice * s, comedi_ins
 
 ==============================================================================
 */
-int icp_multi_insn_bits_di(comedi_device *dev,comedi_subdevice *s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_bits_di(comedi_device *dev,comedi_subdevice *s, comedi_insn *insn, lsampl_t *data)
 {
-	data[1] = inw(dev->iobase + ICP_MULTI_DI);
+	data[1] = readw(dev->iobase + ICP_MULTI_DI);
 
 	return 2;
 }
@@ -502,15 +514,26 @@ int icp_multi_insn_bits_di(comedi_device *dev,comedi_subdevice *s, comedi_insn *
 
 ==============================================================================
 */
-int icp_multi_insn_bits_do(comedi_device *dev,comedi_subdevice *s, comedi_insn *insn,lsampl_t *data)
+static int icp_multi_insn_bits_do(comedi_device *dev,comedi_subdevice *s, comedi_insn *insn,lsampl_t *data)
 {
-	if(data[0]){
-		s->state &= ~data[0];
-		s->state |= (data[0]&data[1]);
-		outw(s->state, dev->iobase + ICP_MULTI_DO);
-	}
-	data[1] = inw(dev->iobase + ICP_MULTI_DI);
+#ifdef ICP_MULTI_EXTDEBUG
+	printk("icp multi EDBG: BGN: icp_multi_insn_bits_do(...)\n");
+#endif
 
+	if (data[0]) {
+		s->state &= ~data[0];
+		s->state |= (data[0] & data[1]);
+
+		printk("Digital outputs = %4x \n", s->state);
+
+		writew(s->state, dev->iobase + ICP_MULTI_DO);
+	}
+
+	data[1] = readw(dev->iobase + ICP_MULTI_DI);
+
+#ifdef ICP_MULTI_EXTDEBUG
+		printk("icp multi EDBG: END: icp_multi_insn_bits_do(...)\n");
+#endif
 	return 2;
 }
 
@@ -533,7 +556,7 @@ int icp_multi_insn_bits_do(comedi_device *dev,comedi_subdevice *s, comedi_insn *
 
 ==============================================================================
 */
-int icp_multi_insn_read_ctr(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_read_ctr(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
 {
         return 0;
 }
@@ -557,7 +580,7 @@ int icp_multi_insn_read_ctr(comedi_device * dev, comedi_subdevice * s, comedi_in
 
 ==============================================================================
 */
-int icp_multi_insn_write_ctr(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
+static int icp_multi_insn_write_ctr(comedi_device * dev, comedi_subdevice * s, comedi_insn *insn, lsampl_t *data)
 {
         return 0;
 }
@@ -578,27 +601,25 @@ int icp_multi_insn_write_ctr(comedi_device * dev, comedi_subdevice * s, comedi_i
 		void *d			Pointer to current device
 		struct pt_regs *regs	Pointer to
 
-	Returns:int			Nmuber of instructions executed
-
 ==============================================================================
 */
-static void interrupt_service_icp_multi(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_service_icp_multi(int irq, void *d, struct pt_regs *regs)
 {
         comedi_device *dev = d;
 	int	int_no;
-	
+
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: BGN: interrupt_service_icp_multi(%d,...)\n",irq);
+	printk("icp multi EDBG: BGN: interrupt_service_icp_multi(%d,...)\n",irq);
 #endif
 
 	// Is this interrupt from our board?
-	int_no = inw(dev->iobase + ICP_MULTI_INT_STAT) & Status_IRQ;
-	if (!int_no) 	
+	int_no = readw(dev->iobase + ICP_MULTI_INT_STAT) & Status_IRQ;
+	if (!int_no)
 		// No, exit
-		return;
+		return IRQ_NONE;
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: interrupt_service_icp_multi() ST: %4x\n",inw(dev->iobase + ICP_MULTI_INT_STAT));
+	printk("icp multi EDBG: interrupt_service_icp_multi() ST: %4x\n",readw(dev->iobase + ICP_MULTI_INT_STAT));
 #endif
 
 	// Determine which interrupt is active & handle it
@@ -626,16 +647,18 @@ static void interrupt_service_icp_multi(int irq, void *d, struct pt_regs *regs)
 	}
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: END: interrupt_service_icp_multi(...)\n");
+	printk("icp multi EDBG: END: interrupt_service_icp_multi(...)\n");
 #endif
+	return IRQ_HANDLED;
 }
 
 
+#if 0
 /*
 ==============================================================================
 
 	Name:	check_channel_list
-	
+
 	Description:
 		This function checks if the channel list, provided by user
 		is built correctly
@@ -651,12 +674,12 @@ static void interrupt_service_icp_multi(int irq, void *d, struct pt_regs *regs)
 
 ==============================================================================
 */
-int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan)
+static int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist, unsigned int n_chan)
 {
         unsigned int i;
     
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG:  check_channel_list(...,%d)\n",n_chan);
+	printk("icp multi EDBG:  check_channel_list(...,%d)\n",n_chan);
 #endif
     	// Check that we at least have one channel to check
 	if (n_chan<1) {
@@ -682,6 +705,7 @@ int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *
 	}
 	return 1;
 }
+#endif
 
 
 
@@ -705,14 +729,14 @@ int check_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *
 
 ==============================================================================
 */
-void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist,
+static void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int *chanlist,
 			unsigned int n_chan)
 {
 	unsigned int i, range, chanprog;
 	unsigned int diff;
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG:  setup_channel_list(...,%d)\n",n_chan);
+	printk("icp multi EDBG:  setup_channel_list(...,%d)\n",n_chan);
 #endif
 	devpriv->act_chanlist_len=n_chan;
 	devpriv->act_chanlist_pos=0;
@@ -750,10 +774,10 @@ void setup_channel_list(comedi_device * dev, comedi_subdevice * s, unsigned int
 		devpriv->AdcCmdStatus |= range;
 
 		/* Output channel, range, mode to ICP Multi*/
-		outw(devpriv->AdcCmdStatus, dev->iobase+ICP_MULTI_ADC_CSR);
+		writew(devpriv->AdcCmdStatus, dev->iobase+ICP_MULTI_ADC_CSR);
 
 #ifdef ICP_MULTI_EXTDEBUG
-		rt_printk("GS: %2d. [%4x]=%4x %4x\n", i, chanprog, range, devpriv->act_chanlist[i]);
+		printk("GS: %2d. [%4x]=%4x %4x\n", i, chanprog, range, devpriv->act_chanlist[i]);
 #endif
 	}
 
@@ -780,11 +804,11 @@ static int icp_multi_reset(comedi_device *dev)
         unsigned int    i;
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp_multi EDBG: BGN: icp_multi_reset(...)\n");
+	printk("icp_multi EDBG: BGN: icp_multi_reset(...)\n");
 #endif
 	// Clear INT enables and requests
-	outw(0, dev->iobase + ICP_MULTI_INT_EN);
-	outw(0x00ff, dev->iobase + ICP_MULTI_INT_STAT);
+	writew(0, dev->iobase + ICP_MULTI_INT_EN);
+	writew(0x00ff, dev->iobase + ICP_MULTI_INT_STAT);
 
 	if (this_board->n_aochan)
 		// Set DACs to 0..5V range and 0V output
@@ -795,23 +819,23 @@ static int icp_multi_reset(comedi_device *dev)
 			devpriv->DacCmdStatus |= (i << 8);
 
 			// Output 0V
-			outw(0, dev->iobase+ICP_MULTI_AO);
+			writew(0, dev->iobase+ICP_MULTI_AO);
 
 			// Set start conversion bit
 			devpriv->DacCmdStatus |= DAC_ST;
 
 			// Output to command / status register
-			outw(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
+			writew(devpriv->DacCmdStatus, dev->iobase+ICP_MULTI_DAC_CSR);
 			
 			// Delay to allow DAC time to recover
-			udelay(1);
+			comedi_udelay(1);
 		}
 	
 	// Digital outputs to 0
-	outw(0, dev->iobase + ICP_MULTI_DO);
+	writew(0, dev->iobase + ICP_MULTI_DO);
 
 #ifdef ICP_MULTI_EXTDEBUG
-	rt_printk("icp multi EDBG: END: icp_multi_reset(...)\n");
+	printk("icp multi EDBG: END: icp_multi_reset(...)\n");
 #endif
 	return 0;
 }
@@ -835,64 +859,78 @@ static int icp_multi_reset(comedi_device *dev)
 */
 static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 {
-	comedi_subdevice *s;
-	int ret, subdev;
-	unsigned short io_addr[5], master,irq;
-	struct pcilst_struct *card=NULL;
-        unsigned int iobase;
-	unsigned char pci_bus, pci_slot, pci_func;
-	
+	comedi_subdevice 	*s;
+	int ret, subdev, n_subdevices;
+	unsigned short 		master,irq;
+	struct pcilst_struct 	*card=NULL;
+        unsigned long 		io_addr[5], iobase;
+	unsigned char 		pci_bus, pci_slot, pci_func;
+
+	printk("icp_multi EDBG: BGN: icp_multi_attach(...)\n");
+
+	// Alocate private data storage space
+	if ((ret=alloc_private(dev, sizeof(icp_multi_private)))<0)
+		return ret;
+
+	// Initialise list of PCI cards in system, if not already done so
 	if (!pci_list_builded) {
-		pci_card_list_init(VENDOR_ID,
+		pci_card_list_init(PCI_VENDOR_ID_ICP,
 #ifdef ICP_MULTI_EXTDEBUG
-						    1);
+						    1
 #else
-						    0);
+						    0
 #endif
-		pci_list_builded=1;
+			);
+			pci_list_builded=1;
 	}
 
-	printk("comedi%d: icp_multi: board=%s",dev->minor,this_board->name);
+	printk("Anne's comedi%d: icp_multi: board=%s", dev->minor, this_board->name);
 
-	if ((card=select_and_alloc_pci_card(VENDOR_ID, this_board->device_id, it->options[0], it->options[1]))==NULL)
+	if ((card=select_and_alloc_pci_card(PCI_VENDOR_ID_ICP, this_board->device_id, it->options[0], it->options[1]))==NULL)
 		return -EIO;
-	
+
 	if ((pci_card_data(card, &pci_bus, &pci_slot, &pci_func, &io_addr[0], &irq, &master))<0) {
 		pci_card_free(card);
 		printk(" - Can't get configuration data!\n");
 		return -EIO;
 	}
-	
+
 	iobase=io_addr[2];
 
-	printk(", b:s:f=%d:%d:%d, io=0x%4x \n", pci_bus, pci_slot, pci_func, iobase);
-	
-        if (check_region(iobase, this_board->iorange) < 0) {
-		pci_card_free(card);
-		printk("I/O port conflict\n");
-		return -EIO;
-        }
+//	if(check_mem_region(iobase, ICP_MULTI_SIZE))
+//	{
+		/* Couldn't allocate io space */
+//		printk(KERN_WARNING "couldn't allocate IO space\n");
+//		return -EIO;
+//	}
+//	request_mem_region(iobase, ICP_MULTI_SIZE, "icp_multi");
+	devpriv->phys_iobase = iobase;
 
-        request_region(iobase, this_board->iorange, "Inova Icp Multi");
-        dev->iobase=iobase;
-    
-	dev->board_name = this_board->name;
+	printk(", b:s:f=%d:%d:%d, io=0x%8lx \n", pci_bus, pci_slot, pci_func, iobase);
 
-	if((ret=alloc_private(dev, sizeof(icp_multi_private)))<0) {
-    		release_region(dev->iobase, this_board->iorange);
-		pci_card_free(card);
+	devpriv->io_addr = ioremap(iobase, ICP_MULTI_SIZE);
+
+	if (devpriv->io_addr == NULL) {
+		printk("ioremap failed.\n");
 		return -ENOMEM;
 	}
-		
-        dev->n_subdevices = 0;
-	if (this_board->n_aichan) dev->n_subdevices++;
-	if (this_board->n_aochan) dev->n_subdevices++;
-	if (this_board->n_dichan) dev->n_subdevices++;
-	if (this_board->n_dochan) dev->n_subdevices++;
-	if (this_board->n_ctrs)	  dev->n_subdevices++;
-	
-        if((ret=alloc_subdevices(dev))<0) {
-    		release_region(dev->iobase, this_board->iorange);
+
+#ifdef ICP_MULTI_EXTDEBUG
+	printk("0x%08lx mapped to %p, ", iobase, devpriv->io_addr);
+#endif
+
+	dev->iobase = (int)devpriv->io_addr;
+
+	dev->board_name = this_board->name;
+
+        n_subdevices = 0;
+	if (this_board->n_aichan) n_subdevices++;
+	if (this_board->n_aochan) n_subdevices++;
+	if (this_board->n_dichan) n_subdevices++;
+	if (this_board->n_dochan) n_subdevices++;
+	if (this_board->n_ctrs)	  n_subdevices++;
+
+        if((ret=alloc_subdevices(dev, n_subdevices))<0) {
 		pci_card_free(card);
     		return ret;
 	}
@@ -902,15 +940,15 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 			if (comedi_request_irq(irq, interrupt_service_icp_multi, SA_SHIRQ, "Inova Icp Multi", dev)) {
 				printk(", unable to allocate IRQ %d, DISABLING IT", irq);
 				irq=0; /* Can't use IRQ */
-			} else {
+			}
+			else
 				printk(", irq=%d", irq);
-			}    
-		} else {
-			printk(", IRQ disabled");
 		}
-	} else {
-		irq=0;
+		else
+			printk(", IRQ disabled");
 	}
+	else
+		irq=0;
 	
         dev->irq = irq;
 
@@ -923,7 +961,8 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 		dev->read_subdev = s;
 		s->type = COMEDI_SUBD_AI;
 		s->subdev_flags = SDF_READABLE|SDF_COMMON|SDF_GROUND;
-		if (this_board->n_aichand) s->subdev_flags |= SDF_DIFF;
+		if (this_board->n_aichand)
+			s->subdev_flags |= SDF_DIFF;
 		s->n_chan = this_board->n_aichan;
 		s->maxdata = this_board->ai_maxdata;
 		s->len_chanlist = this_board->n_aichan;
@@ -935,7 +974,7 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_aochan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_AO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND|SDF_COMMON;
 		s->n_chan = this_board->n_aochan;
 		s->maxdata = this_board->ao_maxdata;
 		s->len_chanlist = this_board->n_aochan;
@@ -948,12 +987,12 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_dichan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_DI;
-		s->subdev_flags = SDF_READABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_READABLE;
 		s->n_chan = this_board->n_dichan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dichan;
 		s->range_table = &range_digital;
-		s->io_bits=0;		/* all bits input */
+		s->io_bits=0;
 		s->insn_bits=icp_multi_insn_bits_di;
 		subdev++;
 	}
@@ -961,12 +1000,12 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_dochan) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_DO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_WRITABLE | SDF_READABLE;
 		s->n_chan = this_board->n_dochan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dochan;
 		s->range_table = &range_digital;
-		s->io_bits=(1 << this_board->n_dochan)-1;	/* all bits output */
+		s->io_bits=(1 << this_board->n_dochan)-1;
 		s->state=0;
 		s->insn_bits=icp_multi_insn_bits_do;
 		subdev++;
@@ -975,7 +1014,7 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_ctrs) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_COUNTER;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND|SDF_COMMON;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND|SDF_COMMON;
 		s->n_chan = this_board->n_ctrs;
 		s->maxdata = 0xffff;
 		s->len_chanlist = this_board->n_ctrs;
@@ -985,10 +1024,14 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 		subdev++;
 	}
 	
-	devpriv->valid=1;
+	devpriv->valid = 1;
 
 	icp_multi_reset(dev);
 
+#ifdef ICP_MULTI_EXTDEBUG
+	printk("icp multi EDBG: END: icp_multi_attach(...)\n");
+#endif
+
 	return 0;
 }
 
@@ -1018,11 +1061,13 @@ static int icp_multi_detach(comedi_device *dev)
 	if (dev->irq)
 		comedi_free_irq(dev->irq,dev);
 
-	if (dev->iobase)
-		release_region(dev->iobase,this_board->iorange);
+	if (dev->iobase) {
+		iounmap(devpriv->io_addr);
+//		release_mem_region(iobase, ICP_MULTI_SIZE);
+	}
 
 	if (pci_list_builded) {
-	    	pci_card_list_cleanup(VENDOR_ID);
+	    	pci_card_list_cleanup(PCI_VENDOR_ID_ICP);
 		pci_list_builded=0;
 	}
 
@@ -1030,11 +1075,3 @@ static int icp_multi_detach(comedi_device *dev)
 	return 0;
 }
 
-/* 
-==============================================================================
-*/
-COMEDI_INITCLEANUP(driver_icp_multi);
-/* 
-==============================================================================
-*/
-
