@@ -81,13 +81,6 @@ options for PCI-20341M:
 
 #include <linux/comedidev.h>
 
-#include <linux/module.h>	/* modularer Kernel */
-#include <linux/kernel.h>	/* printk() */
-#include <linux/errno.h>	/* error codes */
-#include <linux/delay.h>	/* udelay() */
-
-#include <asm/io.h>		/* readb() ... */
-
 
 #define PCI20000_ID			0x1d
 #define PCI20341_ID    			0x77
@@ -216,8 +209,7 @@ static int pci20xxx_attach(comedi_device * dev, comedi_devconfig * it)
 	comedi_subdevice *s;
 	pci20xxx_subdev_private *sdp;
 
-	dev->n_subdevices = 1+PCI20000_MODULES;
-	if ((ret = alloc_subdevices(dev)) < 0)
+	if ((ret = alloc_subdevices(dev, 1 + PCI20000_MODULES)) < 0)
 		return ret;
 	if ((ret = alloc_private(dev, sizeof(pci20xxx_private))) < 0)
 		return ret;
@@ -301,7 +293,7 @@ static int pci20006_init(comedi_device * dev,comedi_subdevice *s,
 
 	/* ao subdevice */
 	s->type = COMEDI_SUBD_AO;
-	s->subdev_flags = SDF_WRITEABLE|SDF_RT;
+	s->subdev_flags = SDF_WRITABLE;
 	s->n_chan = 2;
 	s->len_chanlist = 2;
 	s->insn_read = pci20006_insn_read;
@@ -384,7 +376,7 @@ static int pci20341_init(comedi_device * dev,comedi_subdevice *s,
 
 	/* ai subdevice */
 	s->type = COMEDI_SUBD_AI;
-	s->subdev_flags = SDF_READABLE|SDF_RT;
+	s->subdev_flags = SDF_READABLE;
 	s->n_chan = PCI20341_CHAN_NR;
 	s->len_chanlist = PCI20341_SCANLIST;
 	s->insn_read = pci20341_insn_read;
@@ -460,7 +452,7 @@ static int pci20xxx_dio_init(comedi_device * dev,comedi_subdevice *s)
 {
 
 	s->type = COMEDI_SUBD_DIO;
-	s->subdev_flags = SDF_READABLE | SDF_WRITEABLE;
+	s->subdev_flags = SDF_READABLE | SDF_WRITABLE;
 	s->n_chan = 32;
 	s->insn_bits = pci20xxx_dio_insn_bits;
 	s->insn_config = pci20xxx_dio_insn_config;
