@@ -98,9 +98,6 @@ static int labpc_attach(comedi_device *dev,comedi_devconfig *it);
  */
 #define thisboard ((labpc_board *)dev->board_ptr)
 
-static const int dma_buffer_size = 0xff00;	// size in bytes of dma buffer
-static const int sample_size = 2;	// 2 bytes per sample
-
 static comedi_driver driver_labpc_cs =
 {
 	driver_name: "ni_labpc_cs",
@@ -285,8 +282,8 @@ static dev_link_t *labpc_cs_attach(void)
     link->release.data = (u_long)link;
 
     /* Interrupt setup */
-    link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
-    link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
+    link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_FORCED_PULSE;
+    link->irq.IRQInfo1 = IRQ_INFO2_VALID | IRQ_PULSE_ID;
     if (irq_list[0] == -1)
 		link->irq.IRQInfo2 = irq_mask;
     else for (i = 0; i < 4; i++)
@@ -729,6 +726,7 @@ void labpc_exit_module(void)
 	comedi_driver_unregister(&driver_labpc_cs);
 }
 
+MODULE_LICENSE("GPL");
 module_init( labpc_init_module );
 module_exit( labpc_exit_module );
 
