@@ -26,8 +26,8 @@ Description: National Instruments DAQCard E series
 Author: ds
 Status: works
 Devices: [National Instruments] DAQCard-AI-16XE-50 (ni_mio_cs),
-  DAQCard-AI-16E-4, DAQCard-6062E, DAQCard-6024E
-Updated: Thu, 07 Nov 2002 21:09:40 -0800
+  DAQCard-AI-16E-4, DAQCard-6062E, DAQCard-6024E, DAQCard-6036E
+Updated: Thu Oct 23 19:43:17 CDT 2003
 
 See the notes in the ni_atmio.o driver.
 */
@@ -102,7 +102,7 @@ static ni_board ni_boards[]={
 		name:		"DAQCard-6062E",
 		n_adchan:	16,
 		adbits:		12,
-		ai_fifo_depth:	1024,
+		ai_fifo_depth:	8192,
 		alwaysdither:	0,
 		gainlkup:	ai_gain_16,
 		ai_speed:	2000,
@@ -193,11 +193,6 @@ static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
 	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
 }
 
-#define win_out2(data,addr) do{ \
-	win_out((data)>>16, (addr)); \
-		win_out((data)&0xffff, (addr)+1); \
-}while(0)
-
 #define win_in(addr) __win_in(dev,addr)
 static inline unsigned short __win_in(comedi_device *dev, int addr)
 {
@@ -216,12 +211,6 @@ static inline unsigned short __win_in(comedi_device *dev, int addr)
 	return ret;
 }
 
-#define ao_win_out(a,b) do{ \
-	ni_writew((b),AO_Window_Address_671x); \
-	ni_writew((a),AO_Window_Data_671x); \
-}while(0)
-
-
 typedef struct{
 	dev_link_t *link;
 
@@ -615,6 +604,9 @@ static int ni_getboardtype(comedi_device *dev,dev_link_t *link)
 
 
 #ifdef MODULE
+
+MODULE_LICENSE("GPL");
+
 int init_module(void)
 {
 	servinfo_t serv;
