@@ -29,9 +29,10 @@ Devices: [National Instruments] PCI-MIO-16XE-50 (ni_pcimio),
   PCI-MIO-16XE-10, PXI-6030E, PCI-MIO-16E-1, PCI-MIO-16E-4, PCI-6040E,
   PXI-6040E, PCI-6031E, PCI-6032E, PCI-6033E, PCI-6071E, PCI-6023E,
   PCI-6024E, PCI-6025E, PXI-6025E, PCI-6034E, PCI-6035E, PCI-6052E,
-  PCI-6110, PCI-6111, PCI-6711, PCI-6713, PXI-6071E, PXI-6070E,
-  PXI-6052E, PCI-6036E, PCI-6731, PCI-6733
-Updated: Sat, 16 Mar 2002 17:34:48 -0800
+  PCI-6110, PCI-6111, PCI-6711, PXI-6711, PCI-6713, PXI-6713,
+  PXI-6071E, PXI-6070E,
+  PXI-6052E, PCI-6036E, PCI-6731, PCI-6733, PXI-6733
+Updated: Mon Jan 19 11:00:27 EST 2004
 
 These boards are almost identical to the AT-MIO E series, except that
 they use the PCI bus instead of ISA (i.e., AT).  See the notes for
@@ -44,8 +45,6 @@ By default, the driver uses DMA to transfer analog input data to
 memory.  When DMA is enabled, not all triggering features are
 supported.
 
-Streaming analog output is not supported on PCI-671x and PCI-673x.
-
 PCI IDs are not known for PCI-6731 and PCI-6733.  Digital I/O may not
 work on 673x.
 
@@ -418,7 +417,7 @@ static ni_board ni_boards[]={
 #endif
 	{       device_id:      0x14e0,
 		name:           "pci-6110",
-		n_adchan:       4, 
+		n_adchan:       4,
 		adbits:         12,
 		ai_fifo_depth:  8192,
 		alwaysdither:   0,
@@ -426,10 +425,9 @@ static ni_board ni_boards[]={
 		ai_speed:	200,
 		n_aochan:       2,
 		aobits:         16,
-		ao_671x:	1,
+		reg_type:	ni_reg_611x,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
-		reg_611x:	1,
 		caldac:         {ad8804,ad8804},
 	},
 	{       device_id:      0x14f0,
@@ -442,10 +440,9 @@ static ni_board ni_boards[]={
 		ai_speed:	200,
 		n_aochan:       2,
 		aobits:         16,
-		ao_671x:	1,
+		reg_type:	ni_reg_611x,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
-		reg_611x:	1,
 		caldac:         {ad8804,ad8804},
 	},
 #if 0
@@ -491,9 +488,19 @@ static ni_board ni_boards[]={
 		n_aochan:	4,
 		aobits:         12,
 		ao_unipolar:    0,
-		ao_fifo_depth:  8192,
-		ao_671x:	1,
-		caldac:         {mb88341,mb88341},/* XXX */
+		ao_fifo_depth:  16384, /* data sheet says 8192, but fifo really holds 16384 samples */
+		reg_type:	ni_reg_6711,
+		caldac:         {ad8804_debug},
+	},
+	{       device_id:      0x2b90,
+		name:           "pxi-6711",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	4,
+		aobits:         12,
+		ao_unipolar:    0,
+		ao_fifo_depth:  16384,
+		reg_type:	ni_reg_6711,
+		caldac:         {ad8804_debug},
 	},
 	{       device_id:      0x1870,
 		name:           "pci-6713",
@@ -502,8 +509,8 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		ao_671x:	1,
-		caldac:         {mb88341,mb88341},/* XXX */
+		reg_type:	ni_reg_6713,
+		caldac:         {ad8804_debug,ad8804_debug},
 	},
 	{       device_id:      0x2b80,
 		name:           "pxi-6713",
@@ -512,31 +519,51 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		ao_671x:	1,
-		caldac:         {mb88341,mb88341},/* XXX */
+		reg_type:	ni_reg_6713,
+		caldac:         {ad8804_debug,ad8804_debug},
 	},
-#if 0
-	{       device_id:      0x1880,
+#if 0	/* need device ids */
+	{       device_id:      0x0,
 		name:           "pci-6731",
 		n_adchan:       0, /* no analog input */
 		n_aochan:	4,
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
-		ao_671x:	1,
-		caldac:         {mb88341,mb88341},/* XXX */
+		reg_type:	ni_reg_6711,
+		caldac:         {ad8804_debug},
 	},
-	{       device_id:      0x1870,
+	{       device_id:      0x0,
+		name:           "pxi-6731",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	4,
+		aobits:         16,
+		ao_unipolar:    0,
+		ao_fifo_depth:  8192,
+		reg_type:	ni_reg_6711,
+		caldac:         {ad8804_debug},
+	},
+#endif
+	{       device_id:      0x2410,
 		name:           "pci-6733",
 		n_adchan:       0, /* no analog input */
 		n_aochan:	8,
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		ao_671x:	1,
-		caldac:         {mb88341,mb88341},/* XXX */
+		reg_type:	ni_reg_6713,
+		caldac:         {ad8804_debug,ad8804_debug},
+	},
+	{       device_id:      0x2420,
+		name:           "pxi-6733",
+		n_adchan:       0, /* no analog input */
+		n_aochan:	8,
+		aobits:         16,
+		ao_unipolar:    0,
+		ao_fifo_depth:  16384,
+		reg_type:	ni_reg_6713,
+		caldac:         {ad8804_debug,ad8804_debug},
 	},
-#endif
 	{	device_id:      0x15b0,
 		name:           "pxi-6071e",
 		n_adchan:       64,
@@ -654,11 +681,6 @@ static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
 	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
 }
 
-#define win_out2(data,addr) do{ \
-	win_out((data)>>16, (addr)); \
-	win_out((data)&0xffff, (addr)+1); \
-}while(0)
-
 #define win_in(addr) __win_in(dev,addr)
 static inline unsigned short __win_in(comedi_device *dev, int addr)
 {
@@ -673,23 +695,6 @@ static inline unsigned short __win_in(comedi_device *dev, int addr)
 	return ret;
 }
 
-#define ao_win_out(data,addr) __ao_win_out(dev,data,addr)
-static inline void __ao_win_out( comedi_device *dev, uint16_t data, int addr )
-{
-       unsigned long flags;
-
-       comedi_spin_lock_irqsave(&dev->spinlock,flags);
-       ni_writew(addr,AO_Window_Address_671x);
-       ni_writew(data,AO_Window_Data_671x);
-       comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
-}
-
-#define ao_win_out2(data,addr) do{ \
-       ao_win_out((data)>>16, (addr)); \
-       ao_win_out((data)&0xffff, (addr)+1); \
-}while(0)
-
-
 #define interrupt_pin(a)	0
 #define IRQ_POLARITY 1
 
