@@ -137,7 +137,6 @@
 #define AO_Status_2_Register		6
 
 #define DIO_Parallel_Input_Register	7
-#define DIO_SDIN        			_bit4
 
 #define AI_Command_1_Register		8
 #define AI_Analog_Trigger_Reset			_bit14
@@ -179,6 +178,7 @@
 #define DIO_Parallel_Data_Out(a)                ((a)&0xff)
 #define DIO_Parallel_Data_Mask                  0xff
 #define DIO_SDOUT                               _bit0
+#define DIO_SDIN                                _bit4
 #define DIO_Serial_Data_Out(a)                  (((a)&0xff)<<8)
 #define DIO_Serial_Data_Mask                    0xff00
 
@@ -553,11 +553,13 @@ enum AO_Personal_Bits
 #define G1_Arm                          _bit0
 
 /*Analog_Trigger_Etc_Register*/
-/*This is mainly a counter/timer register */
-#define GPFO_1_Output_Enable		_bit15
-#define GPFO_0_Output_Enable		_bit14
-#define GPFO_0_Output_Select(a)		((a)<<11)
+#define Analog_Trigger_Mode(x) ((x) & 0x7)
+#define Analog_Trigger_Enable _bit3
+#define Analog_Trigger_Drive _bit4
 #define GPFO_1_Output_Select		_bit7
+#define GPFO_0_Output_Select(a)		((a)<<11)
+#define GPFO_0_Output_Enable		_bit14
+#define GPFO_1_Output_Enable		_bit15
 
 /* Additional windowed registers unique to E series */
 
@@ -583,7 +585,6 @@ enum AO_Personal_Bits
 #define ADC_FIFO_Clear			83
 #define DAC_FIFO_Clear			84
 
-
 /* i/o port offsets */
 
 /* 8 bit registers */
@@ -647,32 +648,90 @@ static inline unsigned int AI_CONFIG_CHANNEL( unsigned int channel )
 #define Calibration_Channel_Select_611x	0x1a /* w16 (new) */
 #define ADC_FIFO_Data_611x		0x1c /* r32 (incompatible) */
 #define AI_FIFO_Offset_Load_611x	0x05 /* r8 (new) */
-#define AO_FIFO_Offset_Load_611x       0x13 /* W32? */
 #define DAC_FIFO_Data_611x		0x14 /* w32 (incompatible) */
-#define AO_Window_Addr_611x		0x18 /* w16 */
-#define AO_Window_Data_611x		0x1e /* w16 */
 #define Cal_Gain_Select_611x		0x05 /* w8 (new) */
 
-/* AO Windowed registers */
+#define AO_Window_Address_611x		0x18
+#define AO_Window_Data_611x		0x1e
 
 /* 671x, 611x registers */
 
-#define AO_Window_Address_671x		0x18 /* W 16 */
-#define AO_Window_Data_671x		0x1e /* W 16 */
-
 /* 671xi, 611x windowed ao registers */
-
-#define DACx_Direct_Data_671x(x)	(x) /* W 16 */
-#define AO_Immediate_671x		0x11 /* W 16 */
-#define AO_Timed_611x                   0x10 /* W 16 */
-#define AO_Later_Single_Point_Updates   0x14 /* W 16 */
-#define AO_Waveform_Generation_611x     0x15 /* W 16 */
-#define AO_Misc_611x                    0x16 /* W 16 */
+enum windowed_regs_67xx_61xx
+{
+	AO_Immediate_671x = 0x11, /* W 16 */
+	AO_Timed_611x = 0x10, /* W 16 */
+	AO_FIFO_Offset_Load_611x = 0x13, /* W32 */
+	AO_Later_Single_Point_Updates = 0x14, /* W 16 */
+	AO_Waveform_Generation_611x = 0x15, /* W 16 */
+	AO_Misc_611x = 0x16, /* W 16 */
+	AO_Calibration_Channel_Select_67xx = 0x17, /* W 16 */
+	AO_Configuration_2_67xx = 0x18, /* W 16 */
+	CAL_ADC_Command_67xx = 0x19, /* W 8 */
+	CAL_ADC_Status_67xx = 0x1a, /* R 8 */
+	CAL_ADC_Data_67xx = 0x1b, /* R 16 */
+	CAL_ADC_Config_Data_High_Word_67xx = 0x1c, /* RW 16 */
+	CAL_ADC_Config_Data_Low_Word_67xx = 0x1d, /* RW 16 */
+};
+static inline unsigned int DACx_Direct_Data_671x(int channel)
+{
+	return channel;
+}
 enum AO_Misc_611x_Bits
 {
 	CLEAR_WG = 1,
 };
-
+enum cs5529_configuration_bits
+{
+	CSCFG_CAL_CONTROL_MASK = 0x7,
+	CSCFG_SELF_CAL_OFFSET = 0x1,
+	CSCFG_SELF_CAL_GAIN = 0x2,
+	CSCFG_SELF_CAL_OFFSET_GAIN = 0x3,
+	CSCFG_SYSTEM_CAL_OFFSET = 0x5,
+	CSCFG_SYSTEM_CAL_GAIN = 0x6,
+	CSCFG_DONE = 1 << 3,
+	CSCFG_POWER_SAVE_SELECT = 1 << 4,
+	CSCFG_PORT_MODE = 1 << 5,
+	CSCFG_RESET_VALID = 1 << 6,
+	CSCFG_RESET = 1 << 7,
+	CSCFG_UNIPOLAR = 1 << 12,
+	CSCFG_WORD_RATE_2180_CYCLES = 0x0 << 13,
+	CSCFG_WORD_RATE_1092_CYCLES = 0x1 << 13,
+	CSCFG_WORD_RATE_532_CYCLES = 0x2 << 13,
+	CSCFG_WORD_RATE_388_CYCLES = 0x3 << 13,
+	CSCFG_WORD_RATE_324_CYCLES = 0x4 << 13,
+	CSCFG_WORD_RATE_17444_CYCLES = 0x5 << 13,
+	CSCFG_WORD_RATE_8724_CYCLES = 0x6 << 13,
+	CSCFG_WORD_RATE_4364_CYCLES = 0x7 << 13,
+	CSCFG_WORD_RATE_MASK = 0x7 << 13,
+	CSCFG_LOW_POWER = 1 << 16,
+};
+static inline unsigned int CS5529_CONFIG_DOUT(int output)
+{
+	return 1 << (18 + output);
+}
+static inline unsigned int CS5529_CONFIG_AOUT(int output)
+{
+	return 1 << (22 + output);
+}
+enum cs5529_command_bits
+{
+	CSCMD_POWER_SAVE = 0x1,
+	CSCMD_REGISTER_SELECT_MASK = 0xe,
+	CSCMD_OFFSET_REGISTER = 0x0,
+	CSCMD_GAIN_REGISTER = 0x2,
+	CSCMD_CONFIG_REGISTER = 0x4,
+	CSCMD_READ = 0x10,
+	CSCMD_CONTINUOUS_CONVERSIONS = 0x20,
+	CSCMD_SINGLE_CONVERSION = 0x40,
+	CSCMD_COMMAND = 0x80,
+};
+enum cs5529_status_bits
+{
+	CSS_ADC_BUSY = 0x1,
+	CSS_OSC_DETECT = 0x2, /* indicates adc error */
+	CSS_OVERRANGE = 0x4,
+};
 #define SerDacLd(x)			(0x08<<(x))
 
 /*
@@ -691,6 +750,14 @@ enum mite_dma_channel{
 enum{ ai_gain_16=0, ai_gain_8, ai_gain_14, ai_gain_4, ai_gain_611x };
 enum caldac_enum { caldac_none=0, mb88341, dac8800, dac8043, ad8522,
 	ad8804, ad8842, ad8804_debug };
+enum ni_reg_type {
+	ni_reg_normal = 0x0,
+	ni_reg_611x = 0x1,
+	ni_reg_6711 = 0x2,
+	ni_reg_6713 = 0x4,
+	ni_reg_67xx_mask = 0x6,
+	ni_reg_6xxx_mask = 0x7,
+};
 
 typedef struct ni_board_struct{
 	int device_id;
@@ -711,11 +778,11 @@ typedef struct ni_board_struct{
 	int ao_fifo_depth;
 	int aorangelkup;
 
+	int reg_type;
+
 	unsigned int ao_unipolar : 1;
 	unsigned int has_8255 : 1;
 	unsigned int has_analog_trig : 1;
-	unsigned int ao_671x : 1;
-	unsigned int reg_611x : 1;
 
 	enum caldac_enum caldac[3];
 }ni_board;
