@@ -1,7 +1,7 @@
 /*
  * comedi/drivers/pcl812.c
  *
- * Author: Michal Dobes <majkl@tesnet.cz>
+ * Author: Michal Dobes <dobes@tesnet.cz>
  *
  * hardware driver for Advantech cards
  *  card:   PCL-812, PCL-812PG, PCL-813, PCL-813B
@@ -21,7 +21,7 @@ Description: Advantech PCL-812/PG, PCL-813/B,
              ADLink ACL-8112DG/HG/PG, ACL-8113, ACL-8216,
              ICP DAS A-821PGH/PGL/PGL-NDA, A-822PGH/PGL, A-823PGH/PGL, A-826PG,
              ICP DAS ISO-813
-Author: Michal Dobes <majkl@tesnet.cz>
+Author: Michal Dobes <dobes@tesnet.cz>
 Devices: [Advantech] PCL-812 (pcl812), PCL-812PG (pcl812pg),
   PCL-813 (pcl813), PCL-813B (pcl813b), [ADLink] ACL-8112DG (acl8112dg),
   ACL-8112HG (acl8112hg), ACL-8113 (acl-8113), ACL-8216 (acl8216),
@@ -110,20 +110,12 @@ See the head of the source file pcl812.c for configuration options.
  *     
  */
 
-#include <linux/kernel.h>
-#include <linux/sched.h>
-#include <linux/module.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
-#include <asm/dma.h>
 #include <linux/comedidev.h>
+
+#include <linux/delay.h>
+#include <linux/ioport.h>
+#include <asm/dma.h>
+
 #include "8253.h"
 
 
@@ -438,13 +430,13 @@ static int pcl812_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 	setup_range_channel(dev, s, insn->chanspec, 1); // select channel and renge
 	for(n=0;n<insn->n;n++){
 		outb(255, dev->iobase + PCL812_SOFTTRIG);	/* start conversion */
-		udelay(5);
+		comedi_udelay(5);
 		timeout = 50;	/* wait max 50us, it must finish under 33us */
 		while (timeout--) {
 			hi = inb(dev->iobase + PCL812_AD_HI);
 			if (!(hi & PCL812_DRDY))
 				goto conv_finish;
-			udelay(1);
+			comedi_udelay(1);
 		}
 		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
 		outb(devpriv->mode_reg_int|0, dev->iobase + PCL812_MODE);
@@ -470,12 +462,12 @@ static int acl8216_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 	setup_range_channel(dev, s, insn->chanspec, 1); // select channel and renge
 	for(n=0;n<insn->n;n++){
 		outb(255, dev->iobase + PCL812_SOFTTRIG);	/* start conversion */
-		udelay(5);
+		comedi_udelay(5);
 		timeout = 50;	/* wait max 50us, it must finish under 33us */
 		while (timeout--) {
 			if (!(inb(dev->iobase + ACL8216_STATUS) & ACL8216_DRDY))
 				goto conv_finish;
-			udelay(1);
+			comedi_udelay(1);
 		}
 		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
 		outb(0, dev->iobase + PCL812_MODE);
@@ -772,8 +764,8 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 	} else devpriv->ai_dma=0;
 		
 	devpriv->ai_flags=cmd->flags;
-	devpriv->ai_data_len=s->async->data_len;
-	devpriv->ai_data=s->async->data;
+	devpriv->ai_data_len=s->async->prealloc_bufsz;
+	devpriv->ai_data=s->async->prealloc_buf;
 	if (cmd->stop_src==TRIG_COUNT) { devpriv->ai_scans=cmd->stop_arg; devpriv->ai_neverending=0; }
 				else   { devpriv->ai_scans=0; devpriv->ai_neverending=1; }
 
@@ -856,7 +848,7 @@ static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 /* 
 ==============================================================================
 */
-static void interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *regs)
 {
 	char err=1;
 	unsigned int mask, timeout;
@@ -872,7 +864,7 @@ static void interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *regs)
 			if (!(inb(dev->iobase + ACL8216_STATUS) & ACL8216_DRDY)) {
 			    err=0; break;
 			}
-			udelay(1);
+			comedi_udelay(1);
 		}
 	} else {
 		mask=0x0fff;
@@ -880,50 +872,37 @@ static void interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *regs)
 			if (!(inb(dev->iobase + PCL812_AD_HI) & PCL812_DRDY)) {
 			    err=0; break;
 			}
-			udelay(1);
+			comedi_udelay(1);
 		}
 	}
-	
+
 	if (err) {
 		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D cmd IRQ without DRDY!\n", dev->minor, dev->board_name, dev->iobase);
 		pcl812_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA|COMEDI_CB_ERROR;
 		comedi_event(dev,s,s->async->events);
-		return;
+		return IRQ_HANDLED;
 	}
 
-	*(sampl_t *)(s->async->data+s->async->buf_int_ptr) =
-		((inb(dev->iobase + PCL812_AD_HI) << 8) | inb(dev->iobase + PCL812_AD_LO)) & mask;
-
-	setup_range_channel(dev, s, devpriv->ai_chanlist[s->async->cur_chan], 0); // select channel and renge
+	comedi_buf_put( s->async,
+		((inb(dev->iobase + PCL812_AD_HI) << 8) | inb(dev->iobase + PCL812_AD_LO)) & mask);
 
 	outb(0, dev->iobase + PCL812_CLRINT);	/* clear INT request */
 
-	s->async->buf_int_ptr+=sizeof(sampl_t);
-	s->async->buf_int_count+=sizeof(sampl_t);
-
-	s->async->cur_chan++;
-	if (s->async->cur_chan >= s->async->cmd.chanlist_len) {	/* one scan done */
-		s->async->cur_chan=0;
+	if (s->async->cur_chan == 0 ) {	/* one scan done */
 		devpriv->ai_act_scan++;
-		if (devpriv->ai_eos)
-			s->async->events |= COMEDI_CB_EOS;
 		if (!(devpriv->ai_neverending))
 			if (devpriv->ai_act_scan>=devpriv->ai_scans) {	/* all data sampled */
 				pcl812_ai_cancel(dev,s);
 				s->async->events |= COMEDI_CB_EOA;
 			}
 	}
-	
-	if (s->async->buf_int_ptr >= s->async->data_len) {	/* buffer rollover */
-		s->async->buf_int_ptr = 0;
-		s->async->events |= COMEDI_CB_EOBUF;
-	}
 
 	comedi_event(dev,s,s->async->events);
+	return IRQ_HANDLED;
 }
 
-/* 
+/*
 ==============================================================================
 */
 static void transfer_from_dma_buf(comedi_device *dev,comedi_subdevice *s,
@@ -933,17 +912,10 @@ static void transfer_from_dma_buf(comedi_device *dev,comedi_subdevice *s,
 	
 	s->async->events = 0;
         for (i=len; i; i--) {
-		*(sampl_t *)(s->async->data+s->async->buf_int_ptr)=ptr[bufptr++]; // get one sample
+		comedi_buf_put( s->async, ptr[bufptr++] ); // get one sample
 
-		s->async->buf_int_ptr+=sizeof(sampl_t);
-		s->async->buf_int_count+=sizeof(sampl_t);
-
-		s->async->cur_chan++;
-		if(s->async->cur_chan>=s->async->cmd.chanlist_len){
-			s->async->cur_chan=0;
+		if(s->async->cur_chan == 0){
 			devpriv->ai_act_scan++;
-			if (devpriv->ai_eos)
-				s->async->events |= COMEDI_CB_EOS;
 			if (!devpriv->ai_neverending)
 				if (devpriv->ai_act_scan>=devpriv->ai_scans) {	/* all data sampled */
 					pcl812_ai_cancel(dev,s);
@@ -951,55 +923,46 @@ static void transfer_from_dma_buf(comedi_device *dev,comedi_subdevice *s,
 					break;
 				}
 		}
-
-		if (s->async->buf_int_ptr>=s->async->data_len) { /* buffer rollover */
-	    		s->async->buf_int_ptr=0;
-			s->async->events |= COMEDI_CB_EOBUF;
-			comedi_event(dev,s,s->async->events);
-			s->async->events = 0;
-		}
-
 	}
 
-	s->async->events |= COMEDI_CB_BLOCK;
 	comedi_event(dev,s,s->async->events);
 }
 
 /* 
 ==============================================================================
 */
-static void interrupt_pcl812_ai_dma(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812_ai_dma(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
 	unsigned long dma_flags;
 	int len,bufptr;
-        sampl_t *ptr;
+	sampl_t *ptr;
 
 #ifdef PCL812_EXTDEBUG
 	rt_printk("pcl812 EDBG: BGN: interrupt_pcl812_ai_dma(...)\n");
 #endif
-        ptr=(sampl_t *)devpriv->dmabuf[devpriv->next_dma_buf];
-        len=(devpriv->dmabytestomove[devpriv->next_dma_buf] >> 1) - devpriv->ai_poll_ptr;
+	ptr=(sampl_t *)devpriv->dmabuf[devpriv->next_dma_buf];
+	len=(devpriv->dmabytestomove[devpriv->next_dma_buf] >> 1) - devpriv->ai_poll_ptr;
 
-        devpriv->next_dma_buf=1-devpriv->next_dma_buf;
-        disable_dma(devpriv->dma);
+	devpriv->next_dma_buf=1-devpriv->next_dma_buf;
+	disable_dma(devpriv->dma);
 	set_dma_mode(devpriv->dma, DMA_MODE_READ);
 	dma_flags=claim_dma_lock();
 	set_dma_addr(devpriv->dma, devpriv->hwdmaptr[devpriv->next_dma_buf]);
 	if (devpriv->ai_eos) {
-		set_dma_count(devpriv->dma, devpriv->dmabytestomove[devpriv->next_dma_buf]); 
+		set_dma_count(devpriv->dma, devpriv->dmabytestomove[devpriv->next_dma_buf]);
 	} else {
 		if (devpriv->dma_runs_to_end) { set_dma_count(devpriv->dma, devpriv->dmabytestomove[devpriv->next_dma_buf]); }
-					else { set_dma_count(devpriv->dma, devpriv->last_dma_run); }
-	        devpriv->dma_runs_to_end--;
+		else { set_dma_count(devpriv->dma, devpriv->last_dma_run); }
+		devpriv->dma_runs_to_end--;
 	}
 	release_dma_lock(dma_flags);
 	enable_dma(devpriv->dma);
 
-        outb(0,dev->iobase+PCL812_CLRINT); /* clear INT request */
+	outb(0,dev->iobase+PCL812_CLRINT); /* clear INT request */
 
-        bufptr=devpriv->ai_poll_ptr;
+	bufptr=devpriv->ai_poll_ptr;
 	devpriv->ai_poll_ptr=0;
 
 	transfer_from_dma_buf(dev, s, ptr, bufptr, len);
@@ -1007,21 +970,21 @@ static void interrupt_pcl812_ai_dma(int irq, void *d, struct pt_regs *regs)
 #ifdef PCL812_EXTDEBUG
 	rt_printk("pcl812 EDBG: END: interrupt_pcl812_ai_dma(...)\n");
 #endif
+	return IRQ_HANDLED;
 }
 
-/* 
+/*
 ==============================================================================
 */
-static void interrupt_pcl812(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t interrupt_pcl812(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
 
-	if (devpriv->ai_dma) { interrupt_pcl812_ai_dma(irq, d, regs); }
-			else { interrupt_pcl812_ai_int(irq, d, regs); };
-
+	if (devpriv->ai_dma) { return interrupt_pcl812_ai_dma(irq, d, regs); }
+			else { return interrupt_pcl812_ai_int(irq, d, regs); };
 }
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_ai_poll(comedi_device *dev,comedi_subdevice *s)
@@ -1030,7 +993,7 @@ static int pcl812_ai_poll(comedi_device *dev,comedi_subdevice *s)
         unsigned int top1,top2,i;
 
 	if (!devpriv->ai_dma) return 0; // poll is valid only for DMA transfer
-	
+
 	comedi_spin_lock_irqsave(&dev->spinlock,flags);
 
 	for (i=0; i<10; i++) {
@@ -1054,12 +1017,12 @@ static int pcl812_ai_poll(comedi_device *dev,comedi_subdevice *s)
 
 	transfer_from_dma_buf(dev, s, (void *)devpriv->dmabuf[1-devpriv->next_dma_buf],
 				devpriv->ai_poll_ptr, top2);
-	
+
 	devpriv->ai_poll_ptr=top1; // new buffer position
-	
+
 	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
 
-	return s->async->buf_int_count-s->async->buf_user_count;
+	return s->async->buf_write_count-s->async->buf_read_count;
 }
 
 /*
@@ -1093,7 +1056,7 @@ static void setup_range_channel(comedi_device * dev, comedi_subdevice * s,
 	outb(gain_reg, dev->iobase + PCL812_GAIN); /* select gain */
 
         if (wait) {
-		udelay(devpriv->max_812_ai_mode0_rangewait); // XXX this depends on selected range and can be very long for some high gain ranges!
+		comedi_udelay(devpriv->max_812_ai_mode0_rangewait); // XXX this depends on selected range and can be very long for some high gain ranges!
         }
 }
 
@@ -1108,7 +1071,7 @@ static void start_pacer(comedi_device * dev, int mode, unsigned int divisor1, un
 #endif
         outb(0xb4, dev->iobase + PCL812_CTRCTL);
         outb(0x74, dev->iobase + PCL812_CTRCTL);
-        udelay(1);
+        comedi_udelay(1);
   
         if (mode==1) {
 		outb(divisor2 & 0xff, dev->iobase + PCL812_CTR2);
@@ -1193,17 +1156,17 @@ static void pcl812_reset(comedi_device * dev)
 	case boardPCL813:
 	case boardISO813:
 	case boardACL8113:
-		udelay(5);
+		comedi_udelay(5);
 		break;
 	}
-	udelay(5);
+	comedi_udelay(5);
 #ifdef PCL812_EXTDEBUG
 	rt_printk("pcl812 EDBG: END: pcl812_reset(...)\n");
 #endif
 }
 
 
-/* 
+/*
 ==============================================================================
 */
 static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
@@ -1214,6 +1177,7 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	int dma;
 	unsigned long pages;
 	comedi_subdevice *s;
+	int n_subdevices;
 
 	iobase = it->options[0];
 	printk("comedi%d: pcl812:  board=%s, ioport=0x%03x", dev->minor,
@@ -1293,14 +1257,13 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	}
       no_dma:
 
-	dev->n_subdevices=0;
+	n_subdevices=0;
+	if (this_board->n_aichan > 0) n_subdevices++;
+	if (this_board->n_aochan > 0) n_subdevices++;
+	if (this_board->n_dichan > 0) n_subdevices++;
+	if (this_board->n_dochan > 0) n_subdevices++;
 
-	if (this_board->n_aichan > 0) dev->n_subdevices++;
-	if (this_board->n_aochan > 0) dev->n_subdevices++;
-	if (this_board->n_dichan > 0) dev->n_subdevices++;
-	if (this_board->n_dochan > 0) dev->n_subdevices++;
-
-	if ((ret = alloc_subdevices(dev)) < 0) {
+	if ((ret = alloc_subdevices(dev, n_subdevices)) < 0) {
 		free_resources(dev);
 		return ret;
 	}
@@ -1411,7 +1374,7 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	if (this_board->n_aochan>0) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_AO;
-		s->subdev_flags = SDF_WRITEABLE|SDF_GROUND;
+		s->subdev_flags = SDF_WRITABLE|SDF_GROUND;
 		s->n_chan = this_board->n_aochan;
 		s->maxdata = 0xfff;
 		s->len_chanlist = 1;
@@ -1453,7 +1416,7 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	if (this_board->n_dochan>0) {
 		s = dev->subdevices + subdev;
 		s->type = COMEDI_SUBD_DO;
-		s->subdev_flags = SDF_WRITEABLE;
+		s->subdev_flags = SDF_WRITABLE;
 		s->n_chan = this_board->n_dochan;
 		s->maxdata = 1;
 		s->len_chanlist = this_board->n_dochan;
