@@ -1912,7 +1912,7 @@ static int rtd_ai_cmd (
     if (TRIG_TIMER == cmd->scan_begin_src) {
 					/* scan_begin_arg is in nanoseconds */
 	/* find out how many samples to wait before transferring */
-	if (cmd->flags & COMEDI_EV_SCAN_END) {
+	if (cmd->flags & TRIG_WAKE_EOS) {
 	    /* this may generate un-sustainable interrupt rates */
 	    /* the application is responsible for doing the right thing */
 	    devpriv->transCount = cmd->chanlist_len;
