@@ -44,15 +44,16 @@
 
 #include <linux/comedi.h>
 
+#include <config.h>
+
 
 #define DPRINTK(format, args...)	do{				\
 	if(comedi_debug)printk("comedi: " format , ## args );		\
 } while(0)
 
-#ifndef COMEDI_VERSION_CODE
-#define COMEDI_VERSION_CODE 0x10000
-#define COMEDI_RELEASE "1.0.0"
-#endif
+#define COMEDI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
+#define COMEDI_VERSION_CODE COMEDI_VERSION(COMEDI_MAJORVERSION,COMEDI_MINORVERSION,COMEDI_MICROVERSION)
+#define COMEDI_RELEASE VERSION
 
 #define COMEDI_INITCLEANUP_NOMODULE(x)					\
 	static int __init x ## _init_module(void)			\
@@ -136,9 +137,9 @@ struct comedi_subdevice_struct{
 	//int (*do_unlock)(comedi_device *,comedi_subdevice *);
 
 	/* called when the buffer changes */
-	int (*buf_change)(comedi_device *,comedi_subdevice *s,unsigned long new_size);
+	int (*buf_change)(comedi_device *dev,comedi_subdevice *s,unsigned long new_size);
 
-	void (*munge)( comedi_device *, comedi_subdevice *s, void *data,
+	void (*munge)( comedi_device *dev, comedi_subdevice *s, void *data,
 		unsigned int num_bytes, unsigned int start_chan_index );
 
 	unsigned int state;
@@ -163,6 +164,10 @@ struct comedi_async_struct{
 	unsigned int scan_progress;
 	/* keeps track of where we are in chanlist as for munging */
 	unsigned int munge_chan;
+	/* number of bytes that have been munged */
+	unsigned int munge_count;
+	/* buffer marker for munging */
+	unsigned int munge_ptr;
 
 	unsigned int	events;		/* events that have occurred */
 
@@ -349,17 +354,17 @@ static inline unsigned int bytes_per_sample( const comedi_subdevice *subd )
 int comedi_buf_put(comedi_async *async, sampl_t x);
 int comedi_buf_get(comedi_async *async, sampl_t *x);
 
-unsigned int comedi_buf_write_n_available(comedi_async *async);
+unsigned int comedi_buf_write_n_available(comedi_subdevice *s);
 unsigned int comedi_buf_write_alloc(comedi_async *async, unsigned int nbytes);
 unsigned int comedi_buf_write_alloc_strict(comedi_async *async, unsigned int nbytes);
 void comedi_buf_write_free(comedi_async *async, unsigned int nbytes);
 void comedi_buf_read_free(comedi_async *async, unsigned int nbytes);
-unsigned int comedi_buf_read_n_available(comedi_async *async);
+unsigned int comedi_buf_read_n_available(comedi_subdevice *s);
 void comedi_buf_memcpy_to( comedi_async *async, unsigned int offset, const void *source,
 	unsigned int num_bytes );
 void comedi_buf_memcpy_from( comedi_async *async, unsigned int offset, void *destination,
 	unsigned int num_bytes );
-void comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
+unsigned int comedi_buf_munge( comedi_device *dev, comedi_subdevice *s,
 	unsigned int num_bytes );
 
 //#ifdef CONFIG_COMEDI_RT
